@@ -21,9 +21,9 @@ using std::endl;
 using std::string;
 using std::map;
 
-PekwmAtoms* PekwmAtoms::_instance = NULL;
-IcccmAtoms* IcccmAtoms::_instance = NULL;
-EwmhAtoms* EwmhAtoms::_instance = NULL;
+PekwmAtoms* PekwmAtoms::_instance = 0;
+IcccmAtoms* IcccmAtoms::_instance = 0;
+EwmhAtoms* EwmhAtoms::_instance = 0;
 
 //! @brief PekwmAtoms constructor
 PekwmAtoms::PekwmAtoms(void)
@@ -54,7 +54,7 @@ PekwmAtoms::PekwmAtoms(void)
 //! @brief PekwmAtoms destructor
 PekwmAtoms::~PekwmAtoms(void)
 {
-    _instance = NULL;
+    _instance = 0;
 }
 
 //! @brief IcccmAtoms constructor
@@ -90,7 +90,7 @@ IcccmAtoms::IcccmAtoms(void)
 //! @brief IcccmAtoms destructor
 IcccmAtoms::~IcccmAtoms(void)
 {
-    _instance = NULL;
+    _instance = 0;
 }
 
 //! @brief EwmhAtoms constructor
@@ -105,9 +105,9 @@ EwmhAtoms::EwmhAtoms(void)
                         "_NET_CLIENT_LIST", "_NET_CLIENT_LIST_STACKING",
                         "_NET_NUMBER_OF_DESKTOPS",
                         "_NET_DESKTOP_GEOMETRY", "_NET_DESKTOP_VIEWPORT",
-                        "_NET_CURRENT_DESKTOP", "_NET_ACTIVE_WINDOW",
-                        "_NET_WORKAREA", "_NET_DESKTOP_LAYOUT",
-                        "_NET_SUPPORTING_WM_CHECK",
+                        "_NET_CURRENT_DESKTOP", "_NET_DESKTOP_NAMES",
+                        "_NET_ACTIVE_WINDOW", "_NET_WORKAREA",
+                        "_NET_DESKTOP_LAYOUT", "_NET_SUPPORTING_WM_CHECK",
                         "_NET_CLOSE_WINDOW",
                         "_NET_WM_NAME", "_NET_WM_VISIBLE_NAME",
                         "_NET_WM_ICON", "_NET_WM_DESKTOP",
@@ -151,7 +151,7 @@ EwmhAtoms::EwmhAtoms(void)
 //! @brief EwmhAtoms destructor
 EwmhAtoms::~EwmhAtoms(void)
 {
-    _instance = NULL;
+    _instance = 0;
 }
 
 //! @brief Builds a array of all atoms in the map.
@@ -170,26 +170,40 @@ EwmhAtoms::getAtomArray(void) const
 namespace AtomUtil {
 
 //! @brief Get unknown property
-ulong
-getProperty(Window win, Atom atom, Atom type, ulong expected, uchar** data)
+bool
+getProperty(Window win, Atom atom, Atom type, ulong expected, uchar** data, ulong *actual)
 {
     Atom r_type;
     int r_format, status;
-    ulong read, left;
+    ulong read = 0, left = 0;
 
-    status =
-        XGetWindowProperty(PScreen::instance()->getDpy(), win, atom,
-                           0L, expected, False, type,
-                           &r_type, &r_format, &read, &left, data);
-
-    if ((status != Success) || (read == 0)) {
+    *data = 0;
+    do {
         if (*data) {
             XFree(*data);
-            *data = NULL;
+            *data = 0;
         }
+        expected += left;
+
+        status =
+            XGetWindowProperty(PScreen::instance()->getDpy(), win, atom,
+                               0L, expected, False, type,
+                               &r_type, &r_format, &read, &left, data);
+
+        if (status != Success || type != r_type || read == 0) {
+            if (*data) {
+                XFree(*data);
+                *data = 0;
+            }
+            left = 0;
+        }
+    } while (left);
+
+    if (actual) {
+        * actual = read;
     }
 
-    return (*data != NULL);
+    return (*data != 0);
 }
 
 //! @brief Set XA_ATOM, one value
@@ -228,8 +242,8 @@ setWindows(Window win, Atom atom, Window *values, int size)
 bool
 getLong(Window win, Atom atom, long &value)
 {
-    long *data = NULL;
-    uchar *udata = NULL;
+    long *data = 0;
+    uchar *udata = 0;
 
     if (getProperty(win, atom, XA_CARDINAL, 1L, &udata)) {
         data = reinterpret_cast<long*>(udata);
@@ -254,7 +268,7 @@ setLong(Window win, Atom atom, long value)
 bool
 getString(Window win, Atom atom, string &value)
 {
-    uchar *data = NULL;
+    uchar *data = 0;
 
     if (getProperty(win, atom, XA_STRING, 64L, &data)) {
         value = string((const char*) data);
@@ -271,7 +285,7 @@ bool
 getUtf8String(Window win, Atom atom, std::wstring &value)
 {
     bool status = false;
-    unsigned char *data = NULL;
+    unsigned char *data = 0;
 
     if (getProperty(win, atom, EwmhAtoms::instance()->getAtom(UTF8_STRING),
                     32, &data)) {
@@ -313,7 +327,7 @@ getEwmhPropData(Window win, Atom prop, Atom type, int &num)
     Atom type_ret;
     int format_ret;
     ulong items_ret, after_ret;
-    uchar *prop_data = NULL;
+    uchar *prop_data = 0;
 
     XGetWindowProperty(PScreen::instance()->getDpy(), win, prop, 0, 0x7fffffff,
                        False, type, &type_ret, &format_ret, &items_ret,
