@@ -59,9 +59,9 @@ vector<uint> Client::_clientid_list = vector<uint>();
 
 Client::Client(Window new_client, bool is_new)
     : PWinObj(WindowManager::inst()->getScreen()->getDpy()),
-      _id(0), _size(NULL),
-      _transient(None), _strut(NULL), _icon(NULL),
-      _class_hint(NULL),
+      _id(0), _size(0),
+      _transient(None), _strut(0), _icon(0),
+      _class_hint(0),
       _alive(false), _marked(false),
       _send_focus_message(false), _send_close_message(false),
       _wm_hints_input(true), _cfg_request_lock(false),
@@ -306,13 +306,13 @@ Client::~Client(void)
 
     wo_ref_menu = static_cast<WORefMenu*>(WindowManager::inst()->getMenu("WINDOW"));
     if (this == wo_ref_menu->getWORef()) {
-        wo_ref_menu->setWORef(NULL);
+        wo_ref_menu->setWORef(0);
         wo_ref_menu->unmapAll();
     }
 
     wo_ref_menu = static_cast<WORefMenu*>(WindowManager::inst()->getMenu("DECORMENU"));
     if (this == wo_ref_menu->getWORef()) {
-        wo_ref_menu->setWORef(NULL);
+        wo_ref_menu->setWORef(0);
         wo_ref_menu->unmapAll();
     }
 #endif // MENUS
@@ -343,8 +343,9 @@ Client::~Client(void)
     }
 
     // free names and size hint
-    if (_size)
+    if (_size) {
         XFree(_size);
+    }
 
     removeStrutHint();
 
@@ -354,7 +355,7 @@ Client::~Client(void)
 
     if (_icon) {
         TextureHandler::instance()->returnTexture(_icon);
-        _icon = NULL;
+        _icon = 0;
     }
 
     PScreen::instance()->ungrabServer(true);
@@ -467,6 +468,9 @@ Client::moveResize(int x, int y, uint width, uint height)
     bool request = ((_gm.x != x) || (_gm.y != y)
                   || (_gm.width != width) || (_gm.height != height));
 
+    _gm.x = x;
+    _gm.y = y;
+
     PWinObj::resize(width, height);
 
     if (request) {
@@ -495,20 +499,14 @@ Client::setWorkspace(uint workspace)
 }
 
 //! @brief Gives the Client input focus.
-bool
+void
 Client::giveInputFocus(void)
 {
     if (_wm_hints_input) {
-        return PWinObj::giveInputFocus();
-    } else if (_send_focus_message) {
-        sendXMessage(_window,
-                    IcccmAtoms::instance()->getAtom(WM_PROTOCOLS), NoEventMask,
-                    IcccmAtoms::instance()->getAtom(WM_TAKE_FOCUS),
-                    PScreen::instance()->getLastEventTime());
-        return true;
-    } else {
-        return true;
+        PWinObj::giveInputFocus();
     }
+
+    sendTakeFocusMessage();
 }
 
 //! @brief Reparents and sets _parent member, filtering unmap events
@@ -528,7 +526,7 @@ ActionEvent*
 Client::handleUnmapEvent(XUnmapEvent *ev)
 {
     if ((ev->window != ev->event) && (ev->send_event != true)) {
-        return NULL;
+        return 0;
     }
 
     // ICCCM 4.1.4 advices the window manager to trigger the transition to
@@ -549,20 +547,20 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
     _alive = false;
     delete this;
 
-    return NULL;
+    return 0;
 }
 
 // END - PWinObj interface.
 
 //! @brief Finds the Client which holds the Window w.
 //! @param win Window to search for.
-//! @return Pointer to the client if found, else NULL
+//! @return Pointer to the client if found, else 0
 Client*
 Client::findClient(Window win)
 {
     // Validate input window.
     if ((win == None) || (win == PScreen::instance()->getRoot())) {
-        return NULL;
+        return 0;
     }
 
     list<Client*>::iterator it(_client_list.begin());
@@ -573,7 +571,7 @@ Client::findClient(Window win)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Finds the Client of Window win.
@@ -583,7 +581,7 @@ Client::findClientFromWindow(Window win)
 {
     // Validate input window.
     if (! win || win == PScreen::instance()->getRoot()) {
-        return NULL;
+        return 0;
     }
 
     list<Client*>::iterator it(_client_list.begin());
@@ -593,12 +591,12 @@ Client::findClientFromWindow(Window win)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Finds Client with equal ClassHint.
 //! @param class_hint ClassHint to search for.
-//! @return Client if found, else NULL.
+//! @return Client if found, else 0.
 Client*
 Client::findClientFromHint(const ClassHint *class_hint)
 {
@@ -609,12 +607,12 @@ Client::findClientFromHint(const ClassHint *class_hint)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Finds Client with id.
 //! @param id ID to search for.
-//! @return Client if found, else NULL.
+//! @return Client if found, else 0.
 Client*
 Client::findClientFromID(uint id)
 {
@@ -625,7 +623,7 @@ Client::findClientFromID(uint id)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Checks if the window has any Destroy or Unmap notifys.
@@ -736,7 +734,7 @@ Client::readEwmhHints(void)
 
     // try to figure out what kind of window we are and alter it acordingly
     int items;
-    Atom *atoms = NULL;
+    Atom *atoms = 0;
 
     EwmhAtomName window_type = WINDOW_TYPE;
     atoms = (Atom*) AtomUtil::getEwmhPropData(_window,
@@ -787,8 +785,8 @@ Client::readEwmhHints(void)
         if (win_states.skip_taskbar) _state.skip |= SKIP_TASKBAR;
         if (win_states.skip_pager) _state.skip |= SKIP_PAGER;
         if (win_states.sticky) _sticky = true;
-        if (win_states.above) _layer = LAYER_BELOW;
-        if (win_states.below) _layer = LAYER_ABOVE_DOCK;
+        if (win_states.above) _layer = LAYER_ABOVE_DOCK;
+        if (win_states.below) _layer = LAYER_BELOW;
         if (win_states.fullscreen) _state.fullscreen = true;
     }
 
@@ -878,14 +876,14 @@ Client::readIcon(void)
 
         if (_icon) {
             TextureHandler::instance()->returnTexture(_icon);
-            _icon = NULL;
+            _icon = 0;
         }
     }
 }
 
 //! @brief Tries to find a AutoProp for the current client.
 //! @param type Defaults to 0
-//! @return AutoProperty if any is found, else NULL.
+//! @return AutoProperty if any is found, else 0.
 AutoProperty*
 Client::readAutoprops(uint type)
 {
@@ -899,7 +897,7 @@ Client::readAutoprops(uint type)
                 : ! data->isApplyOn(APPLY_ON_TRANSIENT_ONLY)) {
             applyAutoprops(data);
         } else {
-            data = NULL;
+            data = 0;
         }
     }
 
@@ -981,14 +979,11 @@ Client::getXClientName(void)
 {
     wstring title;
 
-    if (AtomUtil::getUtf8String(_window,
-                              EwmhAtoms::instance()->getAtom(NET_WM_NAME),
-                              title)) {
-    } else {
+    if (! AtomUtil::getUtf8String(_window, EwmhAtoms::instance()->getAtom(NET_WM_NAME), title)) {
         // read X11 property
         XTextProperty text_property;
-        if ((XGetWMName(_dpy, _window, &text_property) == False) ||
-                ! text_property.value || (text_property.nitems == 0)){
+        if ((XGetWMName(_dpy, _window, &text_property) == False)
+            || ! text_property.value || (text_property.nitems == 0)){
             return;
         }
 
@@ -1031,10 +1026,9 @@ Client::titleApplyRule(std::wstring &title)
     _class_hint->title = title;
     TitleProperty *data =
         AutoProperties::instance()->findTitleProperty (_class_hint);
-    _class_hint->title = L"";
 
     if (data) {
-        return data->getTitleRule ().ed_s (title);
+        return data->getTitleRule().ed_s(title);
     } else {
         return false;
     }
@@ -1178,6 +1172,23 @@ Client::configureRequestSend(void)
     XSendEvent(_dpy, _window, false, StructureNotifyMask, (XEvent *) &e);
 }
 
+//! @brief Send a TAKE_FOCUS client message to the client (if requested by it).
+void Client::sendTakeFocusMessage(void)
+{
+    if (_send_focus_message) {
+        {
+            XEvent ev;
+            XChangeProperty(_dpy, RootWindow(_dpy, DefaultScreen(_dpy)), XA_PRIMARY, XA_STRING, 8, PropModeAppend, 0, 0);
+            XWindowEvent(_dpy, RootWindow(_dpy, DefaultScreen(_dpy)), PropertyChangeMask, &ev);
+            PScreen::instance()->setLastEventTime(ev.xproperty.time);
+        }
+        sendXMessage(_window,
+                     IcccmAtoms::instance()->getAtom(WM_PROTOCOLS), NoEventMask,
+                     IcccmAtoms::instance()->getAtom(WM_TAKE_FOCUS),
+                     PScreen::instance()->getLastEventTime());
+    }
+}
+
 //! @brief Grabs a button on the window win
 //! Grabs the button button, with the mod mod and mask mask on the window win
 //! and cursor curs with "all" possible extra modifiers
@@ -1311,7 +1322,7 @@ Client::getMwmHints(Window win)
 {
     Atom real_type; int real_format;
     ulong items_read, items_left;
-    MwmHints *data = NULL;
+    MwmHints *data = 0;
     uchar *udata;
 
     int status =
@@ -1324,10 +1335,10 @@ Client::getMwmHints(Window win)
     if (status == Success) {
         if (items_read < MWM_HINTS_NUM) {
             XFree(udata);
-            udata = NULL;
+            udata = 0;
         }
     } else {
-        udata = NULL;
+        udata = 0;
     }
 
     if (udata) {
@@ -1512,13 +1523,9 @@ void
 Client::getStrutHint(void)
 {
     int num = 0;
-    CARD32 *strut = NULL;
-
-    strut = (CARD32*)
-            AtomUtil::getEwmhPropData(_window,
-                                      EwmhAtoms::instance()->getAtom(NET_WM_STRUT),
-                                      XA_CARDINAL, num);
-
+    long *strut = static_cast<long*>(AtomUtil::getEwmhPropData(_window,
+                                                               EwmhAtoms::instance()->getAtom(NET_WM_STRUT),
+                                                               XA_CARDINAL, num));
     if (strut) {
         if (_strut) {
             PScreen::instance()->removeStrut(_strut);
@@ -1534,7 +1541,7 @@ Client::getStrutHint(void)
     } else if (_strut) {
         PScreen::instance()->removeStrut(_strut);
         delete _strut;
-        _strut = NULL;
+        _strut = 0;
     }
 }
 
@@ -1547,5 +1554,5 @@ Client::removeStrutHint(void)
 
     PScreen::instance()->removeStrut(_strut);
     delete _strut;
-    _strut = NULL;
+    _strut = 0;
 }
