@@ -23,6 +23,7 @@ extern "C" {
 #include "PDecor.hh"
 #include "Frame.hh"
 
+#include "Compat.hh"
 #include "PScreen.hh"
 #include "Config.hh"
 #include "ActionHandler.hh"
@@ -55,14 +56,15 @@ using std::wstring;
 list<Frame*> Frame::_frame_list = list<Frame*>();
 vector<uint> Frame::_frameid_list = vector<uint>();
 
-Frame* Frame::_tag_frame = NULL;
+Frame* Frame::_tag_frame = 0;
 bool Frame::_tag_behind = false;
 
 //! @brief Frame constructor
 Frame::Frame(Client *client, AutoProperty *ap)
     : PDecor(WindowManager::inst()->getScreen()->getDpy(), WindowManager::inst()->getTheme(),
              Frame::getClientDecorName(client)),
-      _id(0), _client(NULL), _class_hint(NULL), _old_decor_state(0)
+      _id(0), _client(0), _class_hint(0),
+      _non_fullscreen_decor_state(0), _non_fullscreen_layer(LAYER_NORMAL)
 {
     // setup basic pointers
     _scr = WindowManager::inst()->getScreen();
@@ -153,7 +155,8 @@ Frame::Frame(Client *client, AutoProperty *ap)
     }
 
     _old_gm = _gm;
-    _old_decor_state = client->getDecorState();
+    _non_fullscreen_decor_state = client->getDecorState();
+    _non_fullscreen_layer = client->getLayer();
 
     _scr->ungrabServer(true); // ungrab and sync
 
@@ -199,7 +202,7 @@ Frame::~Frame(void)
     Workspaces::instance()->remove(this);
     WindowManager::inst()->removeFromFrameList(this);
     if (_tag_frame == this) {
-        _tag_frame = NULL;
+        _tag_frame = 0;
     }
 
     returnFrameID(_id);
@@ -264,10 +267,10 @@ Frame::handleMotionEvent(XMotionEvent *ev)
     // This is true when we have a title button pressed and then we don't want
     // to be able to drag windows around, therefore we ignore the event
     if (_button) {
-        return NULL;
+        return 0;
     }
 
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
     uint button = _scr->getButtonFromState(ev->state);
 
     if (ev->window == getTitleWindow()) {
@@ -276,8 +279,14 @@ Frame::handleMotionEvent(XMotionEvent *ev)
     } else if (ev->window == _client->getWindow()) {
         ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION,
                                             Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME));
-    } else if (ev->subwindow != None) {
+    } else {
         uint pos = getBorderPosition(ev->subwindow);
+
+        // If ev->subwindow wasn't one of the border windows, perhaps ev->window is.
+        if (pos == BORDER_NO_POS) {
+            pos = getBorderPosition(ev->window);
+        }
+        
         if (pos != BORDER_NO_POS) {
             list<ActionEvent> *bl = Config::instance()->getBorderListFromPosition(pos);
             ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION, bl);
@@ -288,7 +297,7 @@ Frame::handleMotionEvent(XMotionEvent *ev)
     if (ae && (ae->threshold > 0)) {
         if (! ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
                                              _pointer_x, _pointer_y, ae->threshold)) {
-            ae = NULL;
+            ae = 0;
         }
     }
 
@@ -299,8 +308,12 @@ Frame::handleMotionEvent(XMotionEvent *ev)
 ActionEvent*
 Frame::handleEnterEvent(XCrossingEvent *ev)
 {
-    ActionEvent *ae = NULL;
-    list<ActionEvent> *al = NULL;
+    // Run event handler to get hoovering to work but ignore action
+    // returned.
+    PDecor::handleEnterEvent(ev);
+
+    ActionEvent *ae = 0;
+    list<ActionEvent> *al = 0;
 
     if (ev->window == getTitleWindow()) {
         al = Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_TITLE_FRAME);
@@ -326,6 +339,10 @@ Frame::handleEnterEvent(XCrossingEvent *ev)
 ActionEvent*
 Frame::handleLeaveEvent(XCrossingEvent *ev)
 {
+    // Run event handler to get hoovering to work but ignore action
+    // returned.
+    PDecor::handleLeaveEvent(ev);
+
     ActionEvent *ae;
 
     MouseActionListName ln = MOUSE_ACTION_LIST_TITLE_FRAME;
@@ -344,7 +361,7 @@ ActionEvent*
 Frame::handleMapRequest(XMapRequestEvent *ev)
 {
     if (! _client || (ev->window != _client->getWindow())) {
-        return NULL;
+        return 0;
     }
 
     if (! _sticky && (_workspace != Workspaces::instance()->getActive())) {
@@ -352,12 +369,12 @@ Frame::handleMapRequest(XMapRequestEvent *ev)
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "Ignoring MapRequest, not on current workspace!" << endl;
 #endif // DEBUG
-        return NULL;
+        return 0;
     }
 
     mapWindow();
 
-    return NULL;
+    return 0;
 }
 
 //! @brief
@@ -372,7 +389,7 @@ Frame::handleUnmapEvent(XUnmapEvent *ev)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 // END - PWinObj interface.
@@ -408,30 +425,37 @@ Frame::removeChild (PWinObj *child, bool do_delete)
     PDecor::removeChild (child, do_delete);
 }
 
-//! @brief Acttivates child in Frame.
+/**
+ * Activates child in Frame, updating it's state and re-loading decor
+ * rules to match updated title.
+ */
 void
 Frame::activateChild (PWinObj *child)
 {
-    // sync the frame state with the client only if we already had a client
-    if (_client && (_client != child))
-        applyState (static_cast<Client*> (child));
+    // Sync the frame state with the client only if we already had a client
+    if (_client && _client != child) {
+        applyState(static_cast<Client*>(child));
+    }
 
-    _client = static_cast<Client*> (child);
-    PDecor::activateChild (child);
+    _client = static_cast<Client*>(child);
+    PDecor::activateChild(child);
 
-    // setShape uses current active child, so we need to activate the child
-    // before setting shape
+    // setShape uses current active child, so we need to activate the
+    // child before setting shape
 #ifdef HAVE_SHAPE
-    if (PScreen::instance ()->hasExtensionShape ()) {
-        _client->setShaped (setShape());
+    if (PScreen::instance()->hasExtensionShape()) {
+        _client->setShaped(setShape());
     }
 #endif // HAVE_SHAPE
 
     if (_focused) {
-        child->giveInputFocus ();
+        child->giveInputFocus();
     }
     
-    Workspaces::instance ()->updateClientStackingList (true, true);
+    // Reload decor rules if needed.
+    handleTitleChange(_client);
+
+    Workspaces::instance()->updateClientStackingList(true, true);
 }
 
 //! @brief
@@ -674,13 +698,13 @@ Frame::setSkip(uint skip)
 
 //! @brief Find Frame with Window
 //! @param win Window to search for.
-//! @return Frame if found, else NULL.
+//! @return Frame if found, else 0.
 Frame*
 Frame::findFrameFromWindow(Window win)
 {
     // Validate input window.
     if ((win == None) || (win == PScreen::instance()->getRoot())) {
-        return NULL;
+        return 0;
     }
 
     list<Frame*>::iterator it(_frame_list.begin());
@@ -690,12 +714,12 @@ Frame::findFrameFromWindow(Window win)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Find Frame with id.
 //! @param id ID to search for.
-//! @return Frame if found, else NULL.
+//! @return Frame if found, else 0.
 Frame*
 Frame::findFrameFromID(uint id)
 {
@@ -706,7 +730,7 @@ Frame::findFrameFromID(uint id)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief
@@ -817,7 +841,6 @@ std::string
 Frame::getClientDecorName(Client *client)
 {
   DecorProperty *prop = WindowManager::inst()->getAutoProperties()->findDecorProperty(client->getClassHint());
-
   return prop ? prop->getName() : PDecor::DEFAULT_DECOR_NAME;
 }
 
@@ -844,7 +867,7 @@ Frame::detachClient(Client *client)
         removeChild(client);
 
         client->move(_gm.x, _gm.y + borderTop());
-        Frame *frame = new Frame(client, NULL);
+        Frame *frame = new Frame(client, 0);
 
         client->setParent(frame);
         client->setWorkspace(Workspaces::instance()->getActive());
@@ -934,7 +957,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
             sw->unmapWindow();
             _scr->ungrabPointer();
 
-            Client *search = NULL;
+            Client *search = 0;
 
             // only group if we have grouping turned on
             if (WindowManager::inst()->isAllowGrouping()) {
@@ -979,7 +1002,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
                 client->move(e.xmotion.x_root, e.xmotion.y_root);
 
-                Frame *frame = new Frame(client, NULL);
+                Frame *frame = new Frame(client, 0);
                 client->setParent(frame);
 
                 // make sure the client ends up on the current workspace
@@ -1145,6 +1168,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
             break;
         case ButtonRelease:
             exit = true;
+            XPutBackEvent(_dpy, &ev);
             break;
         }
     }
@@ -1429,17 +1453,18 @@ Frame::setStateFullscreen(StateAction sa)
     bool lock = _client->setConfigureRequestLock(true);
 
     if (_fullscreen) {
-        if ((_old_decor_state&DECOR_BORDER) != hasBorder()) {
+        if ((_non_fullscreen_decor_state&DECOR_BORDER) != hasBorder()) {
             setBorder(STATE_TOGGLE);
         }
-        if ((_old_decor_state&DECOR_TITLEBAR) != hasTitlebar()) {
+        if ((_non_fullscreen_decor_state&DECOR_TITLEBAR) != hasTitlebar()) {
             setTitlebar(STATE_TOGGLE);
         }
         _gm = _old_gm;
 
     } else {
         _old_gm = _gm;
-        _old_decor_state = _client->getDecorState();
+        _non_fullscreen_decor_state = _client->getDecorState();
+        _non_fullscreen_layer = _client->getLayer();
 
         setBorder(STATE_UNSET);
         setTitlebar(STATE_UNSET);
@@ -1456,6 +1481,12 @@ Frame::setStateFullscreen(StateAction sa)
 
     moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
+    // Re-stack window if fullscreen is above other windows.
+    if (Config::instance()->isFullscreenAbove()) {
+        _layer = _fullscreen ? LAYER_ABOVE_DOCK : _non_fullscreen_layer;
+        raise();
+    }
+    
     _client->setConfigureRequestLock(lock);
     _client->configureRequestSend();
 
@@ -1560,8 +1591,8 @@ Frame::setStateIconified(StateAction sa)
 void
 Frame::setStateTagged(StateAction sa, bool behind)
 {
-    if (ActionUtil::needToggle(sa, (_tag_frame != NULL))) {
-        _tag_frame = (this == _tag_frame) ? NULL : this;
+    if (ActionUtil::needToggle(sa, (_tag_frame != 0))) {
+        _tag_frame = (this == _tag_frame) ? 0 : this;
         _tag_behind = behind;
     }
 }
@@ -1912,23 +1943,9 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
         return; // only handle the active client's events
     }
 
-    // size before position, as we rely on size when gravitating
-    if (! client->isCfgDeny(CFG_DENY_SIZE)) {
-        if ((ev->value_mask&CWWidth) || (ev->value_mask&CWHeight)) {
-            resizeChild(ev->width, ev->height);
-#ifdef HAVE_SHAPE
-            _client->setShaped(setShape());
-#endif // HAVE_SHAPE
-        }
-    }
-
-    if (! client->isCfgDeny(CFG_DENY_POSITION)) {
-        if ((ev->value_mask&CWX) || (ev->value_mask&CWY)) {
-            calcGravityPosition(_client->getXSizeHints()->win_gravity,
-                                ev->x, ev->y, _gm.x, _gm.y);
-            move(_gm.x, _gm.y);
-        }
-    }
+    // Handled geometry, this is handled seperatley due to fullscreen
+    // detection
+    handleConfigureRequestGeometry(ev, client);
 
     // update the stacking
     if (! client->isCfgDeny(CFG_DENY_STACKING)) {
@@ -1965,6 +1982,53 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
     }
 }
 
+/**
+ * Handle size and position part of configure request, detects
+ * fullscreen mode if detection is enabled.
+ */
+void
+Frame::handleConfigureRequestGeometry(XConfigureRequestEvent *ev, Client *client)
+{
+    // Look for fullscreen requests
+    long all_geometry = CWX|CWY|CWWidth|CWHeight;
+    bool is_fullscreen = false;
+    if (Config::instance()->isFullscreenDetect()
+        && ! client->isCfgDeny(CFG_DENY_SIZE)
+        && ! client->isCfgDeny(CFG_DENY_POSITION)
+        && ((ev->value_mask&all_geometry) == static_cast<unsigned>(all_geometry))) {
+
+        Geometry gm_request(ev->x, ev->y, ev->width, ev->height);
+        if (gm_request == _scr->getScreenGeometry()
+             || gm_request == _scr->getHeadGeometry(_scr->getNearestHead(ev->x, ev->y))) {
+
+            is_fullscreen = true;
+            setStateFullscreen(STATE_SET);
+        }
+    }
+
+    if (! is_fullscreen) {
+        // Remove fullscreen state if client changes it size
+        if (Config::instance()->isFullscreenDetect()) {
+            setStateFullscreen(STATE_UNSET);
+        }
+
+        if (! client->isCfgDeny(CFG_DENY_SIZE)
+                && (ev->value_mask & (CWWidth|CWHeight)) ) {
+
+            resizeChild(ev->width, ev->height);
+            _client->setShaped(setShape());
+        }
+
+        if (! client->isCfgDeny(CFG_DENY_POSITION)
+               && (ev->value_mask & (CWX|CWY)) ) {
+
+            calcGravityPosition(_client->getXSizeHints()->win_gravity,
+                                ev->x, ev->y, _gm.x, _gm.y);
+            move(_gm.x, _gm.y);
+        }
+    }
+}
+
 //! @brief
 void
 Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
@@ -2090,16 +2154,36 @@ Frame::handlePropertyChange(XPropertyEvent *ev, Client *client)
         }
     } else if (ev->atom == ewmh->getAtom(NET_WM_STRUT)) {
         client->getStrutHint();
-    } else if (ev->atom == ewmh->getAtom(NET_WM_NAME)) {
-        client->getXClientName();
-        renderTitle();
-    } else if (ev->atom == XA_WM_NAME) {
-//	if (! m_has_extended_net_name)
-        client->getXClientName();
-        renderTitle();
+    } else if (ev->atom == ewmh->getAtom(NET_WM_NAME)
+               || ev->atom == XA_WM_NAME) {
+        handleTitleChange(client);
     } else if (ev->atom == XA_WM_NORMAL_HINTS) {
         client->getWMNormalHints();
     } else if (ev->atom == XA_WM_TRANSIENT_FOR) {
         client->getTransientForHint();
     }
 }
+
+/**
+ * Handle title change, find decoration rules based on changed title
+ * and update if changed.
+ */
+void
+Frame::handleTitleChange(Client *client)
+{
+    // Update title
+    client->getXClientName();
+
+    bool require_render = true;
+    if (client == _client) {
+        string new_decor_name(getClientDecorName(client));
+        if (new_decor_name != _decor_name) {
+            require_render = ! setDecor(new_decor_name);
+        }
+    }
+
+    // Render title if decoration was not updated
+    if (require_render) {
+        renderTitle();
+    }
+}
