@@ -25,6 +25,7 @@ extern "C" {
 #endif // HAVE_SHAPE
 }
 
+#include "Compat.hh"
 #include "Config.hh"
 #include "PWinObj.hh"
 #include "PFont.hh"
@@ -63,12 +64,13 @@ PDecor::Button::Button(Display *dpy, PWinObj *parent, Theme::PDecorButtonData *d
     _gm.height = height;
 
     XSetWindowAttributes attr;
+    attr.event_mask = EnterWindowMask|LeaveWindowMask;
     attr.override_redirect = True;
     _window =
         XCreateWindow(_dpy, _parent->getWindow(),
                       -_gm.width, -_gm.height, _gm.width, _gm.height, 0,
                       CopyFromParent, InputOutput, CopyFromParent,
-                      CWOverrideRedirect, &attr);
+                      CWEventMask|CWOverrideRedirect, &attr);
 
     _bg = ScreenResources::instance()->getPixmapHandler()->getPixmap(_gm.width, _gm.height, PScreen::instance()->getDepth());
 
@@ -92,7 +94,7 @@ PDecor::Button::findAction(XButtonEvent *ev)
         if (it->mod == ev->state && it->sym == ev->button)
             return &*it;
     }
-    return NULL;
+    return 0;
 }
 
 //! @brief Sets the state of the button
@@ -103,29 +105,32 @@ PDecor::Button::setState(ButtonState state)
         return;
     }
 
-    // only update if we don't hoover, we want to be able to turn back
+    // Only update if we don't hoover, we want to be able to turn back
     if (state != BUTTON_STATE_HOOVER) {
         _state = state;
     }
 
-    _data->getTexture(state)->render(_bg, 0, 0, _gm.width, _gm.height);
+    PTexture *texture = _data->getTexture(state);
+    if (texture) {
+        texture->render(_bg, 0, 0, _gm.width, _gm.height);
 
 #ifdef HAVE_SHAPE
-    // Get shape mask
-    bool need_free;
-    Pixmap shape = _data->getTexture(state)->getMask(0, 0, need_free);
-    if (shape != None) {
-        XShapeCombineMask(_dpy, _window, ShapeBounding, 0, 0, shape, ShapeSet);
-        if (need_free) {
-            ScreenResources::instance()->getPixmapHandler()->returnPixmap(shape);
+        // Get shape mask
+        bool need_free;
+        Pixmap shape = _data->getTexture(state)->getMask(0, 0, need_free);
+        if (shape != None) {
+            XShapeCombineMask(_dpy, _window, ShapeBounding, 0, 0, shape, ShapeSet);
+            if (need_free) {
+                ScreenResources::instance()->getPixmapHandler()->returnPixmap(shape);
+            }
+        } else {
+            XRectangle rect = {0 /* x */, 0 /* y */, _gm.width, _gm.height };
+            XShapeCombineRectangles(_dpy, _window, ShapeBounding,
+                                    0, 0, &rect, 1, ShapeSet, YXBanded);
         }
-    } else {
-        XRectangle rect = {0 /* x */, 0 /* y */, _gm.width, _gm.height };
-        XShapeCombineRectangles(_dpy, _window, ShapeBounding,
-                                0, 0, &rect, 1, ShapeSet, YXBanded);
-    }
 #endif
-    clear();
+        clear();
+    }
 }
 
 //! @brief Update visible version of title
@@ -172,13 +177,16 @@ const string PDecor::DEFAULT_DECOR_NAME = string("DEFAULT");
 const string PDecor::DEFAULT_DECOR_NAME_BORDERLESS = string("BORDERLESS");
 const string PDecor::DEFAULT_DECOR_NAME_TITLEBARLESS = string("TITLEBARLESS");
 
+list<PDecor*> PDecor::_pdecor_list = list<PDecor*>();
+
 //! @brief PDecor constructor
 //! @param dpy Display
 //! @param theme Theme
 //! @param decor_name String, if not DEFAULT_DECOR_NAME sets _decor_name_override
 PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
     : PWinObj(dpy),
-      _theme(theme), _child(NULL), _button(NULL),
+      _theme(theme),_decor_name(decor_name),
+      _child(0), _button(0),
       _pointer_x(0), _pointer_y(0),
       _click_x(0), _click_y(0),
       _decor_cfg_keep_empty(false), _decor_cfg_child_move_overloaded(false),
@@ -186,8 +194,7 @@ PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
       _decor_cfg_bpr_al_child(MOUSE_ACTION_LIST_CHILD_OTHER),
       _decor_cfg_bpr_al_title(MOUSE_ACTION_LIST_TITLE_OTHER),
       _maximized_vert(false), _maximized_horz(false),
-      _fullscreen(false), _skip(0),
-      _data(NULL), _decor_name(decor_name),
+      _fullscreen(false), _skip(0), _data(0), 
       _border(true), _titlebar(true), _shaded(false),
       _need_shape(false), _need_client_shape(false),
       _dirty_resized(true), _real_height(1),
@@ -243,11 +250,15 @@ PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
 
     // map title and border windows
     XMapSubwindows(_dpy, _window);
+
+    _pdecor_list.push_back(this);
 }
 
 //! @brief PDecor destructor
 PDecor::~PDecor(void)
 {
+    _pdecor_list.remove(this);
+
     if (_child_list.size() > 0) {
         while (_child_list.size() != 0) {
             removeChild(_child_list.back(), false); // Don't call delete this.
@@ -397,6 +408,14 @@ PDecor::moveResize(int x, int y, uint width, uint height)
     if (_child) {
         _child->moveResize(x + borderLeft(), y + borderTop() + getTitleHeight(),
                           getChildWidth(), getChildHeight());
+
+        // The client window may have its window gravity set to something different
+        // than NorthWestGravity (see Xlib manual chapter 3.2.3). Therefore the 
+        // X server may not move its top left corner along with the decoration.
+        // We correct these cases by calling alignChild(). It is called only for 
+        // _child and not all members of _child_list, because activateChild.*() 
+        // does it itself.
+        alignChild(_child);
     }
 
     // Place and resize title and border
@@ -493,12 +512,11 @@ PDecor::setWorkspace(uint workspace)
 }
 
 //! @brief Gives decor input focus, fails if not mapped or not visible
-bool
+void
 PDecor::giveInputFocus(void)
 {
     if (_mapped && _child) {
-        return _child->giveInputFocus();
-
+        _child->giveInputFocus();
     } else {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
@@ -506,7 +524,6 @@ PDecor::giveInputFocus(void)
              << " *** reverting to root" << endl;
 #endif // DEBUG
         PWinObj::getRootPWinObj()->giveInputFocus();
-        return false;
     }
 }
 
@@ -514,7 +531,7 @@ PDecor::giveInputFocus(void)
 ActionEvent*
 PDecor::handleButtonPress(XButtonEvent *ev)
 {
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
     MouseEventType mb = MOUSE_EVENT_PRESS;
 
     ev->state &= ~Button1Mask & ~Button2Mask & ~Button3Mask
@@ -534,9 +551,9 @@ PDecor::handleButtonPress(XButtonEvent *ev)
         // if the button is used for resizing, we don't want to wait for release
         if (ae && ae->isOnlyAction(ACTION_RESIZE)) {
             _button->setState(_focused ? BUTTON_STATE_FOCUSED : BUTTON_STATE_UNFOCUSED);
-            _button = NULL;
+            _button = 0;
         } else {
-            ae = NULL;
+            ae = 0;
         }
 
     } else {
@@ -570,9 +587,14 @@ PDecor::handleButtonPress(XButtonEvent *ev)
                                                 Config::instance()->getMouseActionList(_decor_cfg_bpr_al_title));
 
             // clicks on the decor border
-        } else if ((ev->subwindow != None) && (_title_wo != ev->subwindow) &&
-                   (ev->window == _window)) {
+        } else {
             uint pos = getBorderPosition(ev->subwindow);
+
+            // If ev->subwindow wasn't one of the border windows, perhaps ev->window is.
+            if (pos == BORDER_NO_POS) {
+                pos = getBorderPosition(ev->window);
+            }
+
             if (pos != BORDER_NO_POS) {
                 list<ActionEvent> *bl = Config::instance()->getBorderListFromPosition(pos);
                 ae = ActionHandler::findMouseAction(ev->button, ev->state, mb, bl);
@@ -587,7 +609,7 @@ PDecor::handleButtonPress(XButtonEvent *ev)
 ActionEvent*
 PDecor::handleButtonRelease(XButtonEvent *ev)
 {
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
     MouseEventType mb = MOUSE_EVENT_RELEASE;
 
     ev->state &= ~Button1Mask & ~Button2Mask & ~Button3Mask
@@ -606,11 +628,11 @@ PDecor::handleButtonRelease(XButtonEvent *ev)
 
             // this is a little hack, resizing isn't wanted on both press and release
             if (ae && ae->isOnlyAction(ACTION_RESIZE)) {
-                ae = NULL;
+                ae = 0;
             }
         }
 
-        _button = NULL;
+        _button = 0;
 
     } else {
 
@@ -650,9 +672,14 @@ PDecor::handleButtonRelease(XButtonEvent *ev)
                                                 Config::instance()->getMouseActionList(_decor_cfg_bpr_al_title));
 
             // clicks on the decor border
-        } else if ((ev->subwindow != None) && (_title_wo != ev->subwindow) &&
-                   (ev->window == _window)) {
+        } else {
             uint pos = getBorderPosition(ev->subwindow);
+            
+            // If ev->subwindow wasn't one of the border windows, perhaps ev->window is.
+            if (pos == BORDER_NO_POS) {
+                pos = getBorderPosition(ev->window);
+            }
+            
             if (pos != BORDER_NO_POS) {
                 list<ActionEvent> *bl = Config::instance()->getBorderListFromPosition(pos);
                 ae = ActionHandler::findMouseAction(ev->button, ev->state, mb, bl);
@@ -673,21 +700,35 @@ PDecor::handleMotionEvent(XMotionEvent *ev)
                                           Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_OTHER));
 }
 
-//! @brief
+/**
+ * Handle enter event, find action and toggle hoover state if enter
+ * was on a button.
+ */
 ActionEvent*
 PDecor::handleEnterEvent(XCrossingEvent *ev)
 {
-    return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
-                                          MOUSE_EVENT_ENTER,
+    PDecor::Button *button = findButton(ev->window);
+    if (button) {
+        button->setState(BUTTON_STATE_HOOVER);
+    }
+
+    return ActionHandler::findMouseAction(BUTTON_ANY, ev->state, MOUSE_EVENT_ENTER,
                                           Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_OTHER));
 }
 
-//! @brief
+/**
+ * Handle leave event, find action and toggle hoover state if leave
+ * was from a button.
+ */
 ActionEvent*
 PDecor::handleLeaveEvent(XCrossingEvent *ev)
 {
-    return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
-                                          MOUSE_EVENT_LEAVE,
+    PDecor::Button *button = findButton(ev->window);
+    if (button) {
+        button->setState(button->getState());
+    }
+
+    return ActionHandler::findMouseAction(BUTTON_ANY, ev->state, MOUSE_EVENT_LEAVE,
                                           Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_OTHER));
 }
 
@@ -810,9 +851,9 @@ PDecor::loadDecor(void)
 void
 PDecor::unloadDecor(void)
 {
-    // Set active button to NULL as it can not be valid after deleting
+    // Set active button to 0 as it can not be valid after deleting
     // the current buttons.
-    _button = NULL;
+    _button = 0;
 
     list<PDecor::Button*>::iterator it(_button_list.begin());
     for (; it != _button_list.end(); ++it) {
@@ -833,7 +874,7 @@ PDecor::findButton(Window win)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief
@@ -841,7 +882,7 @@ PWinObj*
 PDecor::getChildFromPos(int x)
 {
     if (! _child_list.size() || (_child_list.size() != _title_list.size()))
-        return NULL;
+        return 0;
     if (_child_list.size() == 1)
         return _child_list.front();
 
@@ -863,7 +904,7 @@ PDecor::getChildFromPos(int x)
         pos += (*t_it)->getWidth();
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Moves making the child be positioned at x y
@@ -1101,7 +1142,7 @@ PDecor::getChildRel(int off)
              << "PDecor(" << this << ")::getChildRel(" << off << ")" << endl
              << " *** off == 0 or not enough children" << endl;
 #endif // DEBUG
-        return NULL;
+        return 0;
     }
 
     list<PWinObj*>::iterator it(find(_child_list.begin(), _child_list.end(), _child));
@@ -1397,8 +1438,9 @@ PDecor::setShaded(StateAction sa)
         _real_height = _gm.height;
         _gm.height = getTitleHeight();
         // shading in non full width title mode will make border go away
-        if (! _data->getTitleWidthMin())
+        if (! _data->getTitleWidthMin()) {
             _gm.height += borderTop() + borderBottom();
+        }
         _shaded = true;
     }
 
@@ -1472,17 +1514,17 @@ PDecor::renderTitle(void)
         font->setColor(_data->getFontColor(getFocusedState(sel)));
 
         if ((*it)) {
-          PFont::TrimType trim = PFont::FONT_TRIM_MIDDLE;
-          if ((*it)->isCustom() || (*it)->isUserSet()) {
-            trim = PFont::FONT_TRIM_END;
-          }
+            PFont::TrimType trim = PFont::FONT_TRIM_MIDDLE;
+            if ((*it)->isCustom() || (*it)->isUserSet()) {
+                trim = PFont::FONT_TRIM_END;
+            }
 
-          font->draw(_title_bg,
-                     x + _data->getPad(PAD_LEFT), // X position
-                     _data->getPad(PAD_UP), // Y position
-                     (*it)->getVisible(), 0, // Text and max chars
-                     (*it)->getWidth() - pad_horiz, // Available width
-                     trim); // Type of trim
+            font->draw(_title_bg,
+                       x + _data->getPad(PAD_LEFT), // X position
+                       _data->getPad(PAD_UP), // Y position
+                       (*it)->getVisible(), 0, // Text and max chars
+                       (*it)->getWidth() - pad_horiz, // Available width
+                       trim); // Type of trim
         }
 
         // move to next tab (or separator if any)
@@ -1716,12 +1758,13 @@ PDecor::checkEdgeSnap(void)
     }
 }
 
-#ifdef HAVE_SHAPE
+
 //! @brief Set shapes of decor
 //! @return true if shape is on, else false.
 bool
 PDecor::setShape(void)
 {
+#ifdef HAVE_SHAPE
     if (! _child) {
         return false;
     }
@@ -1740,8 +1783,10 @@ PDecor::setShape(void)
     XFree(rect);
 
     return (num > 1);
-}
+#else // !HAVE_SHAPE
+    return false;
 #endif // HAVE_SHAPE
+}
 
 //! @brief
 void
