@@ -33,7 +33,6 @@ extern "C" {
 }
 
 #include "PScreen.hh"
-#include "Config.hh"
 
 using std::cerr;
 using std::endl;
@@ -48,7 +47,7 @@ const uint PScreen::MODIFIER_TO_MASK[] = {
 };
 const uint PScreen::MODIFIER_TO_MASK_NUM = sizeof(PScreen::MODIFIER_TO_MASK[0]) / sizeof(PScreen::MODIFIER_TO_MASK);
 
-PScreen* PScreen::_instance = NULL;
+PScreen* PScreen::_instance = 0;
 
 //! @brief PScreen::Visual constructor.
 //! @param x_visual X Visual to wrap.
@@ -84,16 +83,16 @@ PScreen::PVisual::getShiftPrecFromMask(ulong mask, int &shift, int &prec)
 }
 
 //! @brief PScreen constructor
-PScreen::PScreen(Display *dpy) :
-        _dpy(dpy), _fd(-1),
-        _screen(-1), _depth(-1), _width(0), _height(0),
-        _root(None), _visual(0), _colormap(None),
-        _modifier_map(0),
-        _num_lock(0), _scroll_lock(0),
-        _has_extension_shape(false), _event_shape(-1),
-        _has_extension_xinerama(false),
-        _has_extension_xrandr(false), _event_xrandr(-1),
-        _server_grabs(0), _last_event_time(0), _last_click_id(None)
+PScreen::PScreen(Display *dpy, bool honour_randr)
+    : _dpy(dpy), _honour_randr(honour_randr), _fd(-1),
+      _screen(-1), _depth(-1),
+      _root(None), _visual(0), _colormap(None),
+      _modifier_map(0),
+      _num_lock(0), _scroll_lock(0),
+      _has_extension_shape(false), _event_shape(-1),
+      _has_extension_xinerama(false),
+      _has_extension_xrandr(false), _event_xrandr(-1),
+      _server_grabs(0), _last_event_time(0), _last_click_id(None)
 {
     if (_instance) {
         throw string("PScreen, trying to create multiple instances");
@@ -111,8 +110,8 @@ PScreen::PScreen(Display *dpy) :
     _colormap = DefaultColormap(_dpy, _screen);
     _modifier_map = XGetModifierMapping(_dpy);
 
-    _width = WidthOfScreen(ScreenOfDisplay(_dpy, _screen));
-    _height = HeightOfScreen(ScreenOfDisplay(_dpy, _screen));
+    _screen_gm.width = WidthOfScreen(ScreenOfDisplay(_dpy, _screen));
+    _screen_gm.height = HeightOfScreen(ScreenOfDisplay(_dpy, _screen));
 
 #ifdef HAVE_SHAPE
     {
@@ -153,7 +152,7 @@ PScreen::~PScreen(void) {
         XFreeModifiermap(_modifier_map);
     }
 
-    _instance = NULL;
+    _instance = 0;
 }
 
 //! @brief Get next event using select to avoid signal blocking
@@ -175,7 +174,7 @@ PScreen::getNextEvent(XEvent &ev)
     FD_ZERO(&rfds);
     FD_SET(_fd, &rfds);
 
-    ret = select(_fd + 1, &rfds, NULL, NULL, NULL);
+    ret = select(_fd + 1, &rfds, 0, 0, 0);
     if (ret > 0) {
         XNextEvent(_dpy, &ev);
     }
@@ -266,7 +265,7 @@ void
 PScreen::updateGeometry(uint width, uint height)
 {
 #ifdef HAVE_XRANDR
-  if (! _has_extension_xrandr) {
+  if (! _honour_randr || ! _has_extension_xrandr) {
     return;
   }
 
@@ -275,8 +274,8 @@ PScreen::updateGeometry(uint width, uint height)
   // and strut information is updated.
   initHeads();
 
-  _width = width;
-  _height = height;
+  _screen_gm.width = width;
+  _screen_gm.height = height;
   PWinObj::getRootPWinObj()->resize(width, height);
 
   updateStrut();
@@ -398,6 +397,17 @@ PScreen::getHeadInfo(uint head, Geometry &head_info)
     }
 }
 
+/**
+ * Same as getHeadInfo but returns Geometry instead of filling it in.
+ */
+Geometry
+PScreen::getHeadGeometry(uint head)
+{
+  Geometry gm(_screen_gm);
+  getHeadInfo(head, gm);
+  return gm;
+}
+
 //! @brief Fill information about head and the strut.
 void
 PScreen::getHeadInfoWithEdge(uint num, Geometry &head)
@@ -414,14 +424,14 @@ PScreen::getHeadInfoWithEdge(uint num, Geometry &head)
     head.x += strut_val;
     head.width -= strut_val;  
 
-    strut_val = ((head.x + head.width) == _width) ? std::max(_strut.right, strut.right) : strut.right;
+    strut_val = ((head.x + head.width) == _screen_gm.width) ? std::max(_strut.right, strut.right) : strut.right;
     head.width -= strut_val;
 
     strut_val = (head.y == 0) ? std::max(_strut.top, strut.top) : strut.top;
     head.y += strut_val;
     head.height -= strut_val;
 
-    strut_val = (head.y + head.height == _height) ? std::max(_strut.bottom, strut.bottom) : strut.bottom;
+    strut_val = (head.y + head.height == _screen_gm.height) ? std::max(_strut.bottom, strut.bottom) : strut.bottom;
     head.height -= strut_val;
 }
 
@@ -530,7 +540,7 @@ PScreen::initHeads(void)
         initHeadsXinerama();
 
         if (! _heads.size()) {
-            _heads.push_back(Head(0, 0, _width, _height));
+            _heads.push_back(Head(0, 0, _screen_gm.width, _screen_gm.height));
         }
     }
 }
@@ -561,7 +571,7 @@ void
 PScreen::initHeadsRandr(void)
 {
 #ifdef HAVE_XRANDR
-    if (! _has_extension_xrandr) {
+    if (! _honour_randr || ! _has_extension_xrandr) {
         return;
     }
 
