@@ -37,11 +37,12 @@ using std::map;
 // Theme::PDecorButtonData
 
 //! @brief Theme::PDecorButtonData constructor.
-Theme::PDecorButtonData::PDecorButtonData(void) :
-        _left(false), _width(1), _height(1)
+Theme::PDecorButtonData::PDecorButtonData(void)
+    : ThemeData(),
+      _left(false), _width(1), _height(1)
 {
     for (uint i = 0; i < BUTTON_STATE_NO; ++i) {
-        _texture[i] = NULL;
+        _texture[i] = 0;
     }
 }
 
@@ -55,7 +56,7 @@ Theme::PDecorButtonData::~PDecorButtonData(void)
 //! @param section CfgParser::Entry with button configuration.
 //! @return True if a valid button was parsed.
 bool
-Theme::PDecorButtonData::load (CfgParser::Entry *section)
+Theme::PDecorButtonData::load(CfgParser::Entry *section)
 {
     if (*section == "LEFT") {
         _left = true;
@@ -64,14 +65,12 @@ Theme::PDecorButtonData::load (CfgParser::Entry *section)
     } else {
         return false;
     }
-    
-    section = section->get_section ();
 
     // Get actions.
     ActionEvent ae;
-    CfgParser::Entry *it;
-    for (it = section->get_section_next (); it; it = it->get_section_next ()) {
-        if (Config::instance ()->parseActionEvent (it, ae, BUTTONCLICK_OK, true)) {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
+        if (Config::instance()->parseActionEvent(*it, ae, BUTTONCLICK_OK, true)) {
             _ae_list.push_back (ae);
         }
     }
@@ -83,22 +82,22 @@ Theme::PDecorButtonData::load (CfgParser::Entry *section)
 
         value = section->find_entry ("FOCUSED");
         if (value) {
-            _texture[BUTTON_STATE_FOCUSED] = th->getTexture (value->get_value ());
+            _texture[BUTTON_STATE_FOCUSED] = th->getTexture(value->get_value());
         }
         
         value = section->find_entry ("UNFOCUSED");
         if (value) {
-            _texture[BUTTON_STATE_UNFOCUSED] = th->getTexture (value->get_value ());
+            _texture[BUTTON_STATE_UNFOCUSED] = th->getTexture(value->get_value());
         }
         
         value = section->find_entry ("PRESSED");
         if (value) {
-            _texture[BUTTON_STATE_PRESSED] = th->getTexture (value->get_value ());
+            _texture[BUTTON_STATE_PRESSED] = th->getTexture(value->get_value());
         }
         
         value = section->find_entry ("HOOVER");
         if (value) {
-            _texture[BUTTON_STATE_HOOVER] = th->getTexture (value->get_value ());
+            _texture[BUTTON_STATE_HOOVER] = th->getTexture(value->get_value());
         }
         
         check ();
@@ -115,15 +114,15 @@ Theme::PDecorButtonData::unload(void)
 {
     for (uint i = 0; i < BUTTON_STATE_NO; ++i) {
         TextureHandler::instance()->returnTexture(_texture[i]);
-        _texture[i] = NULL;
+        _texture[i] = 0;
     }
 }
 
-//! @brief Verifies and makes sure no NULL textures exists.
+//! @brief Verifies and makes sure no 0 textures exists.
 void
 Theme::PDecorButtonData::check(void)
 {
-    for (uint i = 0; i < BUTTON_STATE_NO; ++i) {
+    for (uint i = 0; i < (BUTTON_STATE_NO - 1); ++i) {
         if (! _texture[i]) {
             _texture[i] = TextureHandler::instance()->getTexture("EMPTY");
         }
@@ -139,10 +138,15 @@ map<FocusedState, string> Theme::PDecorData::_fs_map = map<FocusedState, string>
 map<BorderPosition, string> Theme::PDecorData::_border_map = map<BorderPosition, string>();
 
 //! @brief Theme::PDecorData constructor.
-Theme::PDecorData::PDecorData(void) :
-        _title_height(0), _title_width_min(0), _title_width_max(100),
-        _title_width_symetric(true), _title_height_adapt(false)
+Theme::PDecorData::PDecorData(const char *name)
+    : ThemeData(),
+      _title_height(0), _title_width_min(0), _title_width_max(100),
+      _title_width_symetric(true), _title_height_adapt(false)
 {
+    if (name) {
+        _name = name;
+    }
+
     // init static data
     if (! _fs_map.size()) {
         _fs_map[FOCUSED_STATE_FOCUSED] = "FOCUSED";
@@ -166,18 +170,18 @@ Theme::PDecorData::PDecorData(void) :
         _pad[i] = 0;
     }
     for (uint i = 0; i < FOCUSED_STATE_NO; ++i) {
-        _texture_tab[i] = NULL;
-        _font[i] = NULL;
-        _font_color[i] = NULL;
+        _texture_tab[i] = 0;
+        _font[i] = 0;
+        _font_color[i] = 0;
     }
     for (uint i = 0; i < FOCUSED_STATE_FOCUSED_SELECTED; ++i) {
-        _texture_main[i] = NULL;
-        _texture_separator[i] = NULL;
+        _texture_main[i] = 0;
+        _texture_separator[i] = 0;
     }
 
     for (uint i = 0; i < FOCUSED_STATE_FOCUSED_SELECTED; ++i) {
         for (uint j = 0; j < BORDER_NO_POS; ++j) {
-            _texture_border[i][j] = NULL;
+            _texture_border[i][j] = 0;
         }
     }
 }
@@ -192,99 +196,87 @@ Theme::PDecorData::~PDecorData(void)
 //! @param section CfgParser::Entry with pdecor configuration.
 //! @return True if a valid pdecor was parsed.
 bool
-Theme::PDecorData::load (CfgParser::Entry *section)
+Theme::PDecorData::load(CfgParser::Entry *section)
 {
+    if (! section) {
+        return false;
+    }
+
     CfgParser::Entry *value;
 
-    _name = section->get_value ();
-    if (! _name.size ()) {
+    _name = section->get_value();
+    if (! _name.size()) {
         cerr << " *** WARNING: no name identifying decor" << endl;
         return false;
     }
 
-    section = section->get_section ();
-
-    CfgParser::Entry *sub, *sub_2;
-    sub = section->find_section ("TITLE");
-    if (! sub) {
+    CfgParser::Entry *title_section = section->find_section("TITLE");
+    if (! title_section) {
         cerr << " *** WARNING: no title section in decor: " << _name << endl;
         return false;
     }
-    sub = sub->get_section ();
 
     TextureHandler *th = TextureHandler::instance(); // convenience
 
     vector<string> tok;
-    list<CfgParserKey*> o_key_list;
-    string o_value_pad, o_value_focused, o_value_unfocused;
+    list<CfgParserKey*> key_list;
+    string value_pad, value_focused, value_unfocused;
+
+    key_list.push_back(new CfgParserKeyInt("HEIGHT", _title_height, 10, 0));
+    key_list.push_back(new CfgParserKeyInt("WIDTHMIN", _title_width_min, 0));
+    key_list.push_back(new CfgParserKeyInt("WIDTHMAX", _title_width_max, 100, 0, 100));
+    key_list.push_back(new CfgParserKeyBool("WIDTHSYMETRIC", _title_width_symetric));
+    key_list.push_back(new CfgParserKeyBool("HEIGHTADAPT", _title_height_adapt));
+    key_list.push_back(new CfgParserKeyString("PAD", value_pad, "0 0 0 0", 7));
+    key_list.push_back(new CfgParserKeyString("FOCUSED", value_focused, "Empty", th->getLengthMin ()));
+    key_list.push_back(new CfgParserKeyString("UNFOCUSED", value_unfocused, "Empty", th->getLengthMin ()));
 
-    o_key_list.push_back (new CfgParserKeyInt ("HEIGHT", _title_height, 10, 0));
-    o_key_list.push_back (new CfgParserKeyInt ("WIDTHMIN", _title_width_min, 0));
-    o_key_list.push_back (new CfgParserKeyInt ("WIDTHMAX", _title_width_max,
-                          100, 0, 100));
-    o_key_list.push_back (new CfgParserKeyBool ("WIDTHSYMETRIC", _title_width_symetric));
-    o_key_list.push_back (new CfgParserKeyBool ("HEIGHTADAPT", _title_height_adapt));
-    o_key_list.push_back (new CfgParserKeyString ("PAD", o_value_pad, "0 0 0 0", 7));
-    o_key_list.push_back (new CfgParserKeyString ("FOCUSED", o_value_focused,
-                          "Empty", th->getLengthMin ()));
-    o_key_list.push_back (new CfgParserKeyString ("UNFOCUSED", o_value_unfocused,
-                          "Empty", th->getLengthMin ()));
     // Free up resources
-    sub->parse_key_values (o_key_list.begin (), o_key_list.end ());
+    title_section->parse_key_values(key_list.begin(), key_list.end());
 
-    for_each (o_key_list.begin (), o_key_list.end (),
-              Util::Free<CfgParserKey*>());
-    o_key_list.clear();
+    for_each (key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    key_list.clear();
 
     // Handle parsed data.
-    _texture_main[FOCUSED_STATE_FOCUSED] = th->getTexture(o_value_focused);
-    _texture_main[FOCUSED_STATE_UNFOCUSED] = th->getTexture(o_value_unfocused);
-    if (Util::splitString(o_value_pad, tok, " \t", 4) == 4) {
+    _texture_main[FOCUSED_STATE_FOCUSED] = th->getTexture(value_focused);
+    _texture_main[FOCUSED_STATE_UNFOCUSED] = th->getTexture(value_unfocused);
+    if (Util::splitString(value_pad, tok, " \t", 4) == 4) {
         for (uint i = 0; i < PAD_NO; ++i)
-            _pad[i] = strtol (tok[i].c_str (), NULL, 10);
+            _pad[i] = strtol(tok[i].c_str (), 0, 10);
     }
 
-    sub_2 = sub->find_section ("TAB");
-    if (sub_2) {
-        sub_2 = sub_2->get_section ();
+    CfgParser::Entry *tab_section = title_section->find_section("TAB");
+    if (tab_section) {
         for (uint i = 0; i < FOCUSED_STATE_NO; ++i) {
-            value = sub_2->find_entry (_fs_map[FocusedState (i)]);
+            value = tab_section->find_entry(_fs_map[FocusedState (i)]);
             if (value) {
-                _texture_tab[i] = th->getTexture (value->get_value ());
+                _texture_tab[i] = th->getTexture(value->get_value ());
             }
         }
     }
 
-    sub_2 = sub->find_section ("SEPARATOR");
-    if (sub_2) {
-        sub_2 = sub_2->get_section ();
-
-        o_key_list.push_back (new CfgParserKeyString ("FOCUSED", o_value_focused,
-                              "Empty", th->getLengthMin ()));
-        o_key_list.push_back (new CfgParserKeyString ("UNFOCUSED", o_value_unfocused,
-                              "Empty", th->getLengthMin ()));
+    CfgParser::Entry *separator_section = title_section->find_section ("SEPARATOR");
+    if (separator_section) {
+        key_list.push_back(new CfgParserKeyString("FOCUSED", value_focused, "Empty", th->getLengthMin()));
+        key_list.push_back(new CfgParserKeyString("UNFOCUSED", value_unfocused, "Empty", th->getLengthMin()));
 
         // Parse data
-        sub_2->parse_key_values (o_key_list.begin (), o_key_list.end ());
+        separator_section->parse_key_values(key_list.begin(), key_list.end());
 
         // Free up resources
-        for_each (o_key_list.begin (), o_key_list.end (),
-                  Util::Free<CfgParserKey*>());
-        o_key_list.clear();
+        for_each (key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+        key_list.clear();
 
         // Handle parsed data.
-        _texture_separator[FOCUSED_STATE_FOCUSED] =
-            th->getTexture(o_value_focused);
-        _texture_separator[FOCUSED_STATE_UNFOCUSED] =
-            th->getTexture(o_value_unfocused);
+        _texture_separator[FOCUSED_STATE_FOCUSED] = th->getTexture(value_focused);
+        _texture_separator[FOCUSED_STATE_UNFOCUSED] = th->getTexture(value_unfocused);
     }
 
 
-    sub_2 = sub->find_section ("FONT");
-    if (sub_2) {
-        sub_2 = sub_2->get_section ();
+    CfgParser::Entry *font_section = title_section->find_section ("FONT");
+    if (font_section) {
         for (uint i = 0; i < FOCUSED_STATE_NO; ++i) {
-            value = sub_2->find_entry (_fs_map[FocusedState(i)]);
+            value = font_section->find_entry(_fs_map[FocusedState(i)]);
             if (value) {
                 _font[i] = FontHandler::instance()->getFont(value->get_value ());
             }
@@ -293,19 +285,18 @@ Theme::PDecorData::load (CfgParser::Entry *section)
         cerr << " *** WARNING: no font section in decor: " << _name << endl;
     }
 
-    sub_2 = sub->find_section ("FONTCOLOR");
-    if (sub_2) {
-        sub_2 = sub_2->get_section ();
+    CfgParser::Entry *fontcolor_section = title_section->find_section ("FONTCOLOR");
+    if (fontcolor_section) {
         for (uint i = 0; i < FOCUSED_STATE_NO; ++i) {
-            value = sub_2->find_entry (_fs_map[FocusedState(i)]);
+            value = fontcolor_section->find_entry (_fs_map[FocusedState(i)]);
             if (value) {
                 _font_color[i] = FontHandler::instance()->getColor(value->get_value ());
             }
         }
     }
 
-    loadButtons (sub->find_section ("BUTTONS"));
-    loadBorder (sub->find_section ("BORDER"));
+    loadButtons (title_section->find_section("BUTTONS"));
+    loadBorder (title_section->find_section("BORDER"));
 
     check();
 
@@ -323,20 +314,20 @@ Theme::PDecorData::unload(void)
         FontHandler::instance()->returnFont(_font[i]);
         FontHandler::instance()->returnColor(_font_color[i]);
 
-        _texture_tab[i] = NULL;
-        _font[i] = NULL;
-        _font_color[i] = NULL;
+        _texture_tab[i] = 0;
+        _font[i] = 0;
+        _font_color[i] = 0;
     }
 
     for (uint i = 0; i < FOCUSED_STATE_FOCUSED_SELECTED; ++i) {
         th->returnTexture(_texture_main[i]);
         th->returnTexture(_texture_separator[i]);
-        _texture_main[i] = NULL;
-        _texture_separator[i] = NULL;
+        _texture_main[i] = 0;
+        _texture_separator[i] = 0;
 
         for (uint j = 0; j < BORDER_NO_POS; ++j) {
             th->returnTexture(_texture_border[i][j]);
-            _texture_border[i][j] = NULL;
+            _texture_border[i][j] = 0;
         }
     }
 
@@ -370,7 +361,6 @@ Theme::PDecorData::loadBorder (CfgParser::Entry *section)
     if (! section) {
         return;
     }
-    section = section->get_section ();
 
     TextureHandler *th = TextureHandler::instance(); // convenience
 
@@ -378,7 +368,6 @@ Theme::PDecorData::loadBorder (CfgParser::Entry *section)
 
     sub = section->find_section ("FOCUSED");
     if (sub) {
-        sub = sub->get_section ();
         for (uint i = 0; i < BORDER_NO_POS; ++i) {
             value = sub->find_entry (_border_map[BorderPosition (i)]);
             if (value) {
@@ -390,7 +379,6 @@ Theme::PDecorData::loadBorder (CfgParser::Entry *section)
 
     sub = section->find_section ("UNFOCUSED");
     if (sub) {
-        sub = sub->get_section ();
         for (uint i = 0; i < BORDER_NO_POS; ++i) {
             value = sub->find_entry (_border_map[BorderPosition (i)]);
             if (value) {
@@ -408,11 +396,15 @@ Theme::PDecorData::loadButtons (CfgParser::Entry *section)
     if (! section) {
         return;
     }
-    section = section->get_section ();
 
-    while ((section = section->get_section_next()) != 0) {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
+        if (! (*it)->get_section()) {
+            continue;
+        }
+
         Theme::PDecorButtonData *btn = new Theme::PDecorButtonData ();
-        if (btn->load (section)) {
+        if (btn->load((*it)->get_section())) {
             _button_list.push_back(btn);
         } else {
             delete btn;
@@ -420,7 +412,7 @@ Theme::PDecorData::loadButtons (CfgParser::Entry *section)
     }
 }
 
-//! @brief Checks for NULL textures, prints warning and sets empty texture
+//! @brief Checks for 0 textures, prints warning and sets empty texture
 void
 Theme::PDecorData::checkTextures(void)
 {
@@ -445,7 +437,7 @@ Theme::PDecorData::checkTextures(void)
     }
 }
 
-//! @brief Checks for NULL fonts, prints warning and sets empty font
+//! @brief Checks for 0 fonts, prints warning and sets empty font
 void
 Theme::PDecorData::checkFonts(void)
 {
@@ -458,7 +450,7 @@ Theme::PDecorData::checkFonts(void)
     }
 }
 
-//! @brief Checks for NULL border PTextures.
+//! @brief Checks for 0 border PTextures.
 void
 Theme::PDecorData::checkBorder(void)
 {
@@ -475,7 +467,7 @@ Theme::PDecorData::checkBorder(void)
     }
 }
 
-//! @brief Checks for NULL colors, prints warning and sets empty color
+//! @brief Checks for 0 colors, prints warning and sets empty color
 void
 Theme::PDecorData::checkColors(void)
 {
@@ -492,16 +484,17 @@ Theme::PDecorData::checkColors(void)
 
 //! @brief PMenuData constructor
 Theme::PMenuData::PMenuData(void)
+    : ThemeData()
 {
     for (uint i = 0; i <= OBJECT_STATE_NO; ++i) {
-        _font[i] = NULL;
-        _color[i] = NULL;
-        _tex_menu[i] = NULL;
-        _tex_item[i] = NULL;
-        _tex_arrow[i] = NULL;
+        _font[i] = 0;
+        _color[i] = 0;
+        _tex_menu[i] = 0;
+        _tex_item[i] = 0;
+        _tex_arrow[i] = 0;
     }
     for (uint i = 0; i < OBJECT_STATE_NO; ++i) {
-        _tex_sep[i] = NULL;
+        _tex_sep[i] = 0;
     }
     for (uint i = 0; i < PAD_NO; ++i) {
         _pad[i] = 0;
@@ -516,38 +509,38 @@ Theme::PMenuData::~PMenuData(void)
 
 //! @brief Parses CfgParser::Entry section, loads and verifies data.
 //! @param section CfgParser::Entry with pmenu configuration.
-void
+bool
 Theme::PMenuData::load(CfgParser::Entry *section)
 {
-    section = section->get_section ();
-
     CfgParser::Entry *value;
     value = section->find_entry ("PAD");
     if (value) {
         vector<string> tok;
         if (Util::splitString (value->get_value (), tok, " \t", 4) == 4) {
             for (int i = 0; i < PAD_NO; ++i) {
-                _pad[i] = strtol (tok[i].c_str(), NULL, 10);
+                _pad[i] = strtol (tok[i].c_str(), 0, 10);
             }
         }
     }
 
     value = section->find_section ("FOCUSED");
     if (value) {
-        loadState (value, OBJECT_STATE_FOCUSED);
+        loadState(value, OBJECT_STATE_FOCUSED);
     }
     
     value = section->find_section ("UNFOCUSED");
     if (value) {
-        loadState (value, OBJECT_STATE_UNFOCUSED);
+        loadState(value, OBJECT_STATE_UNFOCUSED);
     }
     
     value = section->find_section ("SELECTED");
     if (value) {
-        loadState (value, OBJECT_STATE_SELECTED);
+        loadState(value, OBJECT_STATE_SELECTED);
     }
 
     check();
+
+    return true;
 }
 
 //! @brief Unloads data.
@@ -561,16 +554,16 @@ Theme::PMenuData::unload(void)
         TextureHandler::instance()->returnTexture(_tex_item[i]);
         TextureHandler::instance()->returnTexture(_tex_arrow[i]);
 
-        _font[i] = NULL;
-        _color[i] = NULL;
-        _tex_menu[i] = NULL;
-        _tex_item[i] = NULL;
-        _tex_arrow[i] = NULL;
+        _font[i] = 0;
+        _color[i] = 0;
+        _tex_menu[i] = 0;
+        _tex_item[i] = 0;
+        _tex_arrow[i] = 0;
     }
 
     for (uint i = 0; i < OBJECT_STATE_NO; ++i) {
         TextureHandler::instance()->returnTexture(_tex_sep[i]);
-        _tex_sep[i] = NULL;
+        _tex_sep[i] = 0;
     }
 }
 
@@ -607,51 +600,42 @@ Theme::PMenuData::check(void)
 void
 Theme::PMenuData::loadState (CfgParser::Entry *section, ObjectState state)
 {
-    section = section->get_section ();
+    list<CfgParserKey*> key_list;
+    string value_font, value_background, value_item;
+    string value_text, value_arrow, value_separator;
 
-    list<CfgParserKey*> o_key_list;
-    string o_value_font, o_value_background, o_value_item;
-    string o_value_text, o_value_arrow, o_value_separator;
-
-    o_key_list.push_back (new CfgParserKeyString ("FONT", o_value_font));
-    o_key_list.push_back (new CfgParserKeyString ("BACKGROUND", o_value_background,
-                          "Solid #ffffff"));
-    o_key_list.push_back (new CfgParserKeyString ("ITEM", o_value_item,
-                          "Solid #ffffff"));
-    o_key_list.push_back (new CfgParserKeyString ("TEXT", o_value_text,
-                          "Solid #000000"));
-    o_key_list.push_back (new CfgParserKeyString ("ARROW", o_value_arrow,
-                          "Solid #000000"));
-    if (state < OBJECT_STATE_SELECTED)
-    {
-        o_key_list.push_back (new CfgParserKeyString ("SEPARATOR",
-                              o_value_separator,
-                              "Solid #000000"));
+    key_list.push_back(new CfgParserKeyString ("FONT", value_font));
+    key_list.push_back(new CfgParserKeyString ("BACKGROUND", value_background, "Solid #ffffff"));
+    key_list.push_back(new CfgParserKeyString ("ITEM", value_item, "Solid #ffffff"));
+    key_list.push_back(new CfgParserKeyString ("TEXT", value_text, "Solid #000000"));
+    key_list.push_back(new CfgParserKeyString ("ARROW", value_arrow, "Solid #000000"));
+    if (state < OBJECT_STATE_SELECTED) {
+        key_list.push_back(new CfgParserKeyString("SEPARATOR", value_separator, "Solid #000000"));
     }
 
-    section->parse_key_values (o_key_list.begin (), o_key_list.end ());
+    section->parse_key_values(key_list.begin(), key_list.end());
 
-    for_each (o_key_list.begin (), o_key_list.end (),
-              Util::Free<CfgParserKey*>());
+    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
 
     TextureHandler *th = TextureHandler::instance ();
 
     // Handle parsed data.
-    _font[state] = FontHandler::instance ()->getFont (o_value_font);
-    _tex_menu[state] = th->getTexture (o_value_background);
-    _tex_item[state] = th->getTexture (o_value_item);
-    _color[state] = FontHandler::instance ()->getColor (o_value_text);
-    _tex_arrow[state] = th->getTexture (o_value_arrow);
+    _font[state] = FontHandler::instance ()->getFont (value_font);
+    _tex_menu[state] = th->getTexture (value_background);
+    _tex_item[state] = th->getTexture (value_item);
+    _color[state] = FontHandler::instance ()->getColor (value_text);
+    _tex_arrow[state] = th->getTexture (value_arrow);
     if (state < OBJECT_STATE_SELECTED) {
-        _tex_sep[state] = th->getTexture(o_value_separator);
+        _tex_sep[state] = th->getTexture(value_separator);
     }
 }
 
 // Theme::TextDialogData
 
 //! @brief TextDialogData constructor.
-Theme::TextDialogData::TextDialogData(void) :
-        _font(NULL), _color(NULL), _tex(NULL)
+Theme::TextDialogData::TextDialogData(void)
+    : ThemeData(),
+      _font(0), _color(0), _tex(0)
 {
     for (uint i = 0; i < PAD_NO; ++i) {
         _pad[i] = 0;
@@ -666,40 +650,36 @@ Theme::TextDialogData::~TextDialogData(void)
 
 //! @brief Parses CfgParser::Entry section, loads and verifies data.
 //! @param section CfgParser::Entry with textdialog configuration.
-void
-Theme::TextDialogData::load (CfgParser::Entry *section)
+bool
+Theme::TextDialogData::load(CfgParser::Entry *section)
 {
-    section = section->get_section ();
+    list<CfgParserKey*> key_list;
+    string value_font, value_text, value_texture, value_pad;
 
-    list<CfgParserKey*> o_key_list;
-    string o_value_font, o_value_text, o_value_texture, o_value_pad;
+    key_list.push_back(new CfgParserKeyString("FONT", value_font));
+    key_list.push_back(new CfgParserKeyString("TEXT", value_text, "#000000"));
+    key_list.push_back(new CfgParserKeyString("TEXTURE", value_texture, "Solid #ffffff"));
+    key_list.push_back(new CfgParserKeyString("PAD", value_pad, "0 0 0 0", 7));
 
-    o_key_list.push_back (new CfgParserKeyString ("FONT", o_value_font));
-    o_key_list.push_back (new CfgParserKeyString ("TEXT", o_value_text,
-                          "#000000"));
-    o_key_list.push_back (new CfgParserKeyString ("TEXTURE", o_value_texture,
-                          "Solid #ffffff"));
-    o_key_list.push_back (new CfgParserKeyString ("PAD", o_value_pad,
-                          "0 0 0 0", 7));
+    section->parse_key_values(key_list.begin(), key_list.end());
 
-    section->parse_key_values (o_key_list.begin (), o_key_list.end ());
-
-    for_each (o_key_list.begin (), o_key_list.end (),
-              Util::Free<CfgParserKey*>());
+    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
 
     // Handle parsed data.
-    _font = FontHandler::instance ()->getFont (o_value_font);
-    _color = FontHandler::instance ()->getColor (o_value_text);
-    _tex = TextureHandler::instance ()->getTexture (o_value_texture);
+    _font = FontHandler::instance ()->getFont (value_font);
+    _color = FontHandler::instance ()->getColor (value_text);
+    _tex = TextureHandler::instance ()->getTexture (value_texture);
 
     vector<string> tok;
-    if (Util::splitString (o_value_pad, tok, " \t", 4) == 4) {
+    if (Util::splitString(value_pad, tok, " \t", 4) == 4) {
         for (uint i = 0; i < PAD_NO; ++i) {
-            _pad[i] = strtol(tok[i].c_str(), NULL, 10);
+            _pad[i] = strtol(tok[i].c_str(), 0, 10);
         }
     }
 
     check();
+
+    return true;
 }
 
 //! @brief Unloads data.
@@ -710,9 +690,9 @@ Theme::TextDialogData::unload(void)
     FontHandler::instance()->returnColor(_color);
     TextureHandler::instance()->returnTexture(_tex);
 
-    _font = NULL;
-    _tex = NULL;
-    _color = NULL;
+    _font = 0;
+    _tex = 0;
+    _color = 0;
 }
 
 //! @brief Check data properties, prints warning and tries to fix.
@@ -737,9 +717,10 @@ Theme::TextDialogData::check(void)
  * WorkspaceIndicatorData constructor
  */
 Theme::WorkspaceIndicatorData::WorkspaceIndicatorData(void)
-  : font(0), font_color(0), texture_background(0), 
-    texture_workspace(0), texture_workspace_act(0),
-    edge_padding(0), workspace_padding(0)
+    : ThemeData(),
+      font(0), font_color(0), texture_background(0), 
+      texture_workspace(0), texture_workspace_act(0),
+      edge_padding(0), workspace_padding(0)
 {
 }
 
@@ -754,11 +735,9 @@ Theme::WorkspaceIndicatorData::~WorkspaceIndicatorData(void)
 /**
  * Load theme data and check.
  */
-void
-Theme::WorkspaceIndicatorData::load(CfgParser::Entry *cs)
+bool
+Theme::WorkspaceIndicatorData::load(CfgParser::Entry *section)
 {
-    cs = cs->get_section();
-
     list<CfgParserKey*> key_list;
 
     string value_font, value_color, value_tex_bg;
@@ -772,8 +751,8 @@ Theme::WorkspaceIndicatorData::load(CfgParser::Entry *cs)
     key_list.push_back(new CfgParserKeyInt("EDGEPADDING", edge_padding, 5, 0));
     key_list.push_back(new CfgParserKeyInt("WORKSPACEPADDING", workspace_padding, 2, 0));
 
-    cs->parse_key_values(key_list.begin(), key_list.end());
-    for_each (key_list.begin (), key_list.end (), Util::Free<CfgParserKey*>());  
+    section->parse_key_values(key_list.begin(), key_list.end());
+    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());  
 
     font = FontHandler::instance()->getFont(value_font);
     font_color = FontHandler::instance()->getColor(value_color);
@@ -782,6 +761,8 @@ Theme::WorkspaceIndicatorData::load(CfgParser::Entry *cs)
     texture_workspace_act = TextureHandler::instance()->getTexture(value_tex_ws_act);
 
     check();
+
+    return true;
 }
 
 /**
@@ -832,16 +813,91 @@ Theme::WorkspaceIndicatorData::check(void)
     }
 }
 
+#ifdef HARBOUR
+// HarbourData
+
+/**
+ * HarbourData constructor.
+ */
+Theme::HarbourData::HarbourData(void)
+    : ThemeData(),
+      _texture(0)
+{
+}
+
+/**
+ * HarbourData destructor, unload data.
+ */
+Theme::HarbourData::~HarbourData(void)
+{
+    unload();
+}
+
+/**
+ * Load harbour data and validate state, unloading previously loaded
+ * data if any.
+ */
+bool
+Theme::HarbourData::load(CfgParser::Entry *section)
+{
+    CfgParser::Entry *value;
+
+    value = section->find_entry ("TEXTURE");
+    if (value) {
+        _texture = TextureHandler::instance()->getTexture (value->get_value());
+    }
+
+    check();
+
+    return true;
+}
+
+
+/**
+ * Unload harbour data.
+ */
+void
+Theme::HarbourData::unload(void)
+{
+    if (_texture) {
+        TextureHandler::instance()->returnTexture(_texture);
+        _texture = 0;        
+    }
+}
+
+
+/**
+ * Check state of harbour data.
+ */
+void
+Theme::HarbourData::check(void)
+{
+    if (! _texture) {
+        _texture = TextureHandler::instance()->getTexture("EMPTY");
+    }
+}
+
+#endif // HARBOUR
+
 // Theme
 
 //! @brief Theme constructor
-Theme::Theme(PScreen *scr) :
-        _scr(scr), _image_handler(NULL),
-        _is_loaded(false), _invert_gc(None)
+Theme::Theme(PScreen *scr)
+    : _scr(scr), _image_handler(0), _theme_mtime(0),
+      _is_loaded(false), _invert_gc(None)
 {
     // image handler
     _image_handler = new ImageHandler();
 
+    // Map between theme sections and ThemeData structures.
+    _section_data_map["MENU"] = &_menu_data;
+    _section_data_map["STATUS"] = &_status_data;
+    _section_data_map["CMDDIALOG"] = &_cmd_d_data;
+    _section_data_map["WORKSPACEINDICATOR"] = &_workspace_indicator_data;
+#ifdef HARBOUR
+    _section_data_map["HARBOUR"] = &_harbour_data;
+#endif // HARBOUR
+
     // window gc's
     XGCValues gv;
     gv.function = GXinvert;
@@ -868,51 +924,59 @@ Theme::~Theme(void)
     delete _image_handler;
 }
 
-//! @brief Loads the "ThemeFile", unloads any previous loaded theme.
-void
+/**
+ * Re-loads theme if needed, clears up previously used resources.
+ */
+bool
 Theme::load(const std::string &dir)
 {
+    string norm_dir(dir);
+    if (dir.size() && dir.at(dir.size() - 1) != '/') {
+        norm_dir.append("/");
+    }
+    string theme_file(norm_dir + string("theme"));
+
+    if (! Util::requireReload(_theme_path, theme_file, _theme_mtime)) {
+        return false;
+    }
+
     if (_is_loaded) {
         unload();
     }
-    
-    CfgParser theme;
 
-    _theme_dir = dir;
-    if (_theme_dir.size ()) {
-        if (_theme_dir.at (_theme_dir.size() - 1) != '/') {
-            _theme_dir.append("/");
-        }
-    } else {
+    _theme_dir = norm_dir;
+    if (! _theme_dir.size()) {
         cerr << " *** WARNING: empty theme directory name, using default." << endl;
         _theme_dir = DATADIR "/pekwm/themes/default/";
     }
 
-    string theme_file = _theme_dir + string("theme");
-    if (! theme.parse(theme_file, CfgParserSource::SOURCE_FILE)) {
+    bool theme_ok = true;
+    CfgParser theme;
+
+    if (! theme.parse(theme_file)) {
         _theme_dir = DATADIR "/pekwm/themes/default/";
-        theme_file = _theme_dir + string ("theme");
-        if (! theme.parse(theme_file, CfgParserSource::SOURCE_FILE)) {
-            cerr << " *** WARNING: couldn't load " << _theme_dir
-                 << " or default theme." << endl;
+        _theme_path = _theme_dir + string("theme");
+        if (! theme.parse(_theme_path)) {
+            cerr << " *** WARNING: couldn't load " << _theme_dir << " or default theme." << endl;
+            theme_ok = false;
         }
     }
 
-    CfgParser::Entry *section, *value;
+    // Setup quirks and requirements before parsing.
+    if (theme_ok) {
+        loadThemeRequire(theme);
+    }
 
     // Set image basedir.
     _image_handler->setDir(_theme_dir);
 
     // Load decor data.
-    section = theme.get_entry_root ()->find_section ("PDECOR");
+    CfgParser::Entry *section = theme.get_entry_root()->find_section("PDECOR");
     if (section) {
-        // Get section data, don't need name or value.
-        section = section->get_section ();
-
-        //section = section->get_section ();
-        while ((section = section->get_section_next ()) != 0) {
+        CfgParser::iterator it(section->begin());
+        for (; it != section->end(); ++it) {
             Theme::PDecorData *data = new Theme::PDecorData();
-            if (data->load (section)) {
+            if (data->load((*it)->get_section())) {
                 _pdecordata_map[data->getName()] = data;
             } else {
                 delete data;
@@ -920,94 +984,75 @@ Theme::load(const std::string &dir)
         }
     }
 
-    if (! getPDecorData ("DEFAULT")) {
-        cerr << " *** WARNING: theme doesn't contain any DEFAULT decor." << endl;
-
+    if (! getPDecorData("DEFAULT")) {
         // Create DEFAULT decor, let check fill it up with empty but non-null data.
-        Theme::PDecorData *data = new Theme::PDecorData();
-        data->setName("DEFAULT");
-        data->check();
-        _pdecordata_map[data->getName()] = data;
+        cerr << " *** WARNING: theme doesn't contain any DEFAULT decor." << endl;
+        Theme::PDecorData *decor_data = new Theme::PDecorData("DEFAULT");
+        decor_data->check();
+        _pdecordata_map["DEFAULT"] = decor_data;
     }
 
-    // Load menu data.
-    section = theme.get_entry_root ()->find_section ("MENU");
-    if (section) {
-        _menu_data.load(section);
-    } else {
-        cerr << " *** WARNING: missing menu section" << endl;
-        _menu_data.check ();
-    }
-
-    // Load StatusWindow data.
-    section = theme.get_entry_root ()->find_section ("STATUS");
-    if (section) {
-        _status_data.load (section);
-    } else {
-        cerr << " *** WARNING: missing status section" << endl;
-        _status_data.check ();
-    }
-
-    // Load CmdDialog data.
-    section = theme.get_entry_root ()->find_section ("CMDDIALOG");
-    if (section) {
-        _cmd_d_data.load (section);
-    } else {
-        cerr << " *** WARNING: missing cmddialog section" << endl;
-        _cmd_d_data.check ();
-    }
-
-    // Load WorkspaceIndicator data.
-    section = theme.get_entry_root()->find_section("WORKSPACEINDICATOR");
-    if (section) {
-      _workspace_indicator_data.load(section);
-    } else {
-      cerr << " *** WARNING: missing workspaceindicator section" << endl;
-      _workspace_indicator_data.check();
-    }
-
-#ifdef HARBOUR
-    // Load Harbour texture.
-    section = theme.get_entry_root ()->find_section ("HARBOUR");
-    if (section) {
-        section = section->get_section ();
-
-        value = section->find_entry ("TEXTURE");
-        if (value) {
-            _harbour_texture = TextureHandler::instance ()->getTexture (value->get_value ());
+    map<string, ThemeData*>::iterator it(_section_data_map.begin());
+    for (; it != _section_data_map.end(); ++it) {
+        section = theme.get_entry_root()->find_section(it->first);
+        if (section) {
+            it->second->load(section);
+        } else {
+            cerr << " *** WARNING: missing " << it->first << " section!" << endl;
+            it->second->check();
         }
-    } else {
-        cerr << " *** WARNING: missing harbour section" << endl;
     }
-    
-    if (! _harbour_texture) {
-        cerr << " *** WARNING: missing harbour texture" << endl;
-        _harbour_texture = TextureHandler::instance ()->getTexture("EMPTY");
-    }
-#endif // HARBOUR
 
     _is_loaded = true;
+
+    return true;
 }
 
-//! @brief Unloads all pixmaps, fonts, gc and colors allocated by the theme.
+/**
+ * Load template quirks.
+ */
 void
-Theme::unload(void) {
-    map<string, Theme::PDecorData*>::iterator it(_pdecordata_map.begin());
-    for (; it != _pdecordata_map.end(); ++it) {
-        delete it->second;
+Theme::loadThemeRequire(CfgParser &theme_cfg)
+{
+    CfgParser::Entry *section;
+
+    // Look for requires section, 
+    section = theme_cfg.get_entry_root()->find_section("REQUIRE");
+    if (section) {
+        list<CfgParserKey*> key_list;
+        bool value_templates;
+
+        key_list.push_back(new CfgParserKeyBool("TEMPLATES", value_templates, false));
+        section->parse_key_values(key_list.begin(), key_list.end());
+        for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+
+        // Re-load configuration with templates enabled.
+        if (value_templates) {
+            string theme_path(_theme_path);
+            theme_cfg.clear(true);
+            theme_cfg.parse(theme_path, CfgParserSource::SOURCE_FILE, true);
+        }
+    }
+}
+
+/**
+ * Unload theme data.
+ */
+void
+Theme::unload(void)
+{
+    // Unload decors
+    map<string, Theme::PDecorData*>::iterator p_it(_pdecordata_map.begin());
+    for (; p_it != _pdecordata_map.end(); ++p_it) {
+        delete p_it->second;
     }
     _pdecordata_map.clear();
 
-#ifdef HARBOUR
-    TextureHandler::instance()->returnTexture(_harbour_texture);
-    _harbour_texture = NULL;
-#endif // HARBOUR
-
-    // unload menu
-    _menu_data.unload();
-
-    _status_data.unload();
-    _cmd_d_data.unload();
+    // Unload theme data
+    map<string, Theme::ThemeData*>::iterator d_it(_section_data_map.begin());
+    for (; d_it != _section_data_map.end(); ++d_it) {
+        d_it->second->unload();
+    }
 
     _is_loaded = false;
 }
