@@ -95,6 +95,8 @@ using std::wstring;
 
 // Static initializers
 const string WindowManager::_wm_name = string("pekwm");
+const ulong WindowManager::EXPECTED_DESKTOP_NAMES_LENGTH = 256;
+
 WindowManager *WindowManager::_inst = 0;
 
 #ifdef MENUS
@@ -139,7 +141,7 @@ extern "C" {
             is_signal_int_term = true;
             break;
         case SIGCHLD:
-            wait(NULL);
+            wait(0);
             break;
         case SIGALRM:
             // Do nothing, just used to break out of waiting
@@ -399,17 +401,22 @@ WindowManager::RootWO::handleLeaveEvent(XCrossingEvent *ev)
 
 // WindowManager
 
-void WindowManager::start(const std::string &command_line, const std::string &config_file) {
-    if (_inst)
+/**
+ * Create window manager instance and run main routine.
+ */
+void
+WindowManager::start(const std::string &command_line, const std::string &config_file) {
+    if (_inst) {
         delete _inst;
-    
+    }
     _inst = new WindowManager(command_line, config_file);
 
     _inst->setupDisplay();
 
     _inst->scanWindows();
     Frame::resetFrameIDs();
-
+    _inst->setDesktopNames();
+    
     // add all frames to the MRU list
     _inst->_mru_list.resize(Frame::frame_size());
     copy(Frame::frame_begin(), Frame::frame_end (), _inst->_mru_list.begin());
@@ -425,21 +432,24 @@ void WindowManager::destroy() {
 
 //! @brief Constructor for WindowManager class
 WindowManager::WindowManager(const std::string &command_line, const std::string &config_file) :
-        _screen(NULL), _screen_resources(NULL),
-        _keygrabber(NULL),
-        _config(NULL), _color_handler(NULL),
-        _font_handler(NULL), _texture_handler(NULL),
-        _theme(NULL), _action_handler(NULL),
-        _autoproperties(NULL), _workspaces(NULL),
+        _screen(0), _screen_resources(0),
+        _keygrabber(0),
+        _config(0), _color_handler(0),
+        _font_handler(0), _texture_handler(0),
+        _theme(0), _action_handler(0),
+        _autoproperties(0), _workspaces(0),
 #ifdef HARBOUR
-        _harbour(NULL),
+        _harbour(0),
 #endif // HARBOUR
-        _cmd_dialog(NULL),
-        _status_window(NULL), _workspace_indicator(NULL),
+        _cmd_dialog(0),
+        _status_window(0), _workspace_indicator(0),
+#ifdef MENUS
+        _menu_mtime(0),
+#endif // MENUS
         _command_line(command_line),
         _startup(false), _shutdown(false), _reload(false),
-        _allow_grouping(true), _root_wo(NULL),
-        _pekwm_atoms(NULL), _icccm_atoms(NULL), _ewmh_atoms(NULL)
+        _allow_grouping(true), _root_wo(0),
+        _pekwm_atoms(0), _icccm_atoms(0), _ewmh_atoms(0)
 {
     struct sigaction act;
 
@@ -448,15 +458,16 @@ WindowManager::WindowManager(const std::string &command_line, const std::string
     act.sa_mask = sigset_t();
     act.sa_flags = SA_NOCLDSTOP | SA_NODEFER;
 
-    sigaction(SIGTERM, &act, NULL);
-    sigaction(SIGINT, &act, NULL);
-    sigaction(SIGHUP, &act, NULL);
-    sigaction(SIGCHLD, &act, NULL);
-    sigaction(SIGALRM, &act, NULL);
+    sigaction(SIGTERM, &act, 0);
+    sigaction(SIGINT, &act, 0);
+    sigaction(SIGHUP, &act, 0);
+    sigaction(SIGCHLD, &act, 0);
+    sigaction(SIGALRM, &act, 0);
 
     // construct
     _config = new Config();
     _config->load(config_file);
+    _config->loadMouseConfig(_config->getMouseConfigFile());
 }
 
 //! @brief WindowManager destructor
@@ -464,8 +475,10 @@ WindowManager::~WindowManager(void)
 {
     cleanup();
 
-    if (_root_wo)
-        delete _root_wo;
+    delete _cmd_dialog;
+    delete _status_window;
+    delete _workspace_indicator;
+    delete _root_wo;
 
 #ifdef HARBOUR
     if (_harbour) {
@@ -473,14 +486,6 @@ WindowManager::~WindowManager(void)
     }
 #endif // HARBOUR
 
-    if (_cmd_dialog) {
-        delete _cmd_dialog;
-    }
-    if (_status_window)
-        delete _status_window;
-    if (_workspace_indicator)
-        delete _workspace_indicator;
-
     if (_action_handler)
         delete _action_handler;
     if (_autoproperties)
@@ -591,7 +596,7 @@ WindowManager::cleanup(void)
 void
 WindowManager::setupDisplay(void)
 {
-    Display *dpy = XOpenDisplay(NULL);
+    Display *dpy = XOpenDisplay(0);
     if (! dpy) {
         cerr << "Can not open display!" << endl
              << "Your DISPLAY variable currently is set to: "
@@ -601,15 +606,14 @@ WindowManager::setupDisplay(void)
 
     XSetErrorHandler(handleXError);
 
-    _screen = new PScreen(dpy);
+    _screen = new PScreen(dpy, _config->isHonourRandr());
     _color_handler = new ColorHandler(dpy);
     _font_handler = new FontHandler();
     _texture_handler = new TextureHandler();
     _action_handler = new ActionHandler();
 
-    // setup the font trimming
-    wstring trim_title(Util::to_wide_str(_config->getTrimTitle()));
-    PFont::setTrimString(trim_title);
+    // Setup the font trimming
+    PFont::setTrimString(_config->getTrimTitle());
 
     // load colors, fonts
     _screen_resources = new ScreenResources();
@@ -852,104 +856,147 @@ WindowManager::screenEdgeMapUnmap(void)
 void
 WindowManager::doReload(void)
 {
-    // unique client names state
-    bool old_client_unique_name = _config->getClientUniqueName();
-    string old_client_unique_name_pre =
-        _config->getClientUniqueNamePre();
-    string old_client_unique_name_post =
-        _config->getClientUniqueNamePost();
+    doReloadConfig();
+    doReloadTheme();
+    doReloadMouse();
+    doReloadKeygrabber();
+    doReloadAutoproperties();
 
+#ifdef MENUS
+    doReloadMenus();
+#endif // MENUS
 #ifdef HARBOUR
-    // I do not want to restack or rearrange if nothing has changed.
-    uint old_harbour_placement = _config->getHarbourPlacement();
-    bool old_harbour_stacking = _config->isHarbourOntop();
-    int old_harbour_head_nr = _config->getHarbourHead();
+    doReloadHarbour();
 #endif // HARBOUR
 
-    _config->load(_config->getConfigFile()); // reload the config
-    _autoproperties->load(); // reload autoprops config
+    setDesktopNames();
 
-    // update what might have changed in the cfg toouching the hints
+    _reload = false;
+}
+
+/**
+ * Reload main config file.
+ */
+void
+WindowManager::doReloadConfig(void)
+{
+    // If any of these changes, re-fetch of all names is required
+    bool old_client_unique_name = _config->getClientUniqueName();
+    string old_client_unique_name_pre = _config->getClientUniqueNamePre();
+    string old_client_unique_name_post = _config->getClientUniqueNamePost();
+
+    // Reload configuration
+    if (! _config->load(_config->getConfigFile())) {
+        return;
+    }
+
+    // Update what might have changed in the cfg touching the hints
     _workspaces->setSize(_config->getWorkspaces());
     _workspaces->setPerRow(_config->getWorkspacesPerRow());
+    _workspaces->setNames();
 
-    // flush pixmap cache and set size
+    // Flush pixmap cache and set size
     _screen_resources->getPixmapHandler()->setCacheSize(_config->getScreenPixmapCacheSize());
 
-    // set the font title trim before reloading the themes
-    wstring trim_title(Util::to_wide_str(_config->getTrimTitle()));
-    PFont::setTrimString(trim_title);
+    // Set the font title trim before reloading the themes
+    PFont::setTrimString(_config->getTrimTitle());
 
-    // update the ClientUniqueNames if needed
+    // Update the ClientUniqueNames if needed
     if ((old_client_unique_name != _config->getClientUniqueName()) ||
         (old_client_unique_name_pre != _config->getClientUniqueNamePre()) ||
         (old_client_unique_name_post != _config->getClientUniqueNamePost())) {
-        for_each(Client::client_begin(), Client::client_end(),
-                 mem_fun(&Client::getXClientName));
+        for_each(Client::client_begin(), Client::client_end(), mem_fun(&Client::getXClientName));
     }
 
-    // reload the theme
-    _theme->load(_config->getThemeFile());
-
-    // resize the screen edge
+    // Resize the screen edge
     screenEdgeResize();
     screenEdgeMapUnmap();
 
-    // regrab the buttons on the client windows
-    for_each(Client::client_begin(), Client::client_end(),
-             mem_fun(&Client::grabButtons));
+    _screen->updateStrut();
+}
 
-    // reload the keygrabber
-    _keygrabber->load(_config->getKeyFile());
+/**
+ * Reload theme file and update decorations.
+ */
+void
+WindowManager::doReloadTheme(void)
+{
+    // Reload the theme
+    if (! _theme->load(_config->getThemeFile())) {
+        return;
+    }
+
+    // Reload the themes on all decors
+    for_each(PDecor::pdecor_begin(), PDecor::pdecor_end(), mem_fun(&PDecor::loadDecor));
+}
+
+/**
+ * Reload mouse configuration and re-grab buttons on all windows.
+ */
+void
+WindowManager::doReloadMouse(void)
+{
+    if (! _config->loadMouseConfig(_config->getMouseConfigFile())) {
+        return;
+    }
+
+    for_each(Client::client_begin(), Client::client_end(), mem_fun(&Client::grabButtons));
+}
+
+/**
+ * Reload keygrabber configuration and re-grab keys on all windows.
+ */
+void
+WindowManager::doReloadKeygrabber(void)
+{
+    // Reload the keygrabber
+    if (! _keygrabber->load(_config->getKeyFile())) {
+        return;
+    }
 
     _keygrabber->ungrabKeys(_screen->getRoot());
     _keygrabber->grabKeys(_screen->getRoot());
 
-    // regrab keys
+    // Regrab keys and buttons
     list<Client*>::iterator c_it(Client::client_begin());
     for (; c_it != Client::client_end(); ++c_it) {
+        (*c_it)->grabButtons();
         _keygrabber->ungrabKeys((*c_it)->getWindow());
         _keygrabber->grabKeys((*c_it)->getWindow());
     }
+}
 
-    // update the status window and cmd dialog theme
-    _cmd_dialog->loadDecor();
-    _status_window->loadDecor();
-    _workspace_indicator->loadDecor();
-
-    // reload the themes on the frames
-    for_each(Frame::frame_begin(), Frame::frame_end(),
-             mem_fun(&PDecor::loadDecor));
+/**
+ * Reload autoproperties.
+ */
+void
+WindowManager::doReloadAutoproperties(void)
+{
+    if (! _autoproperties->load()) {
+        return;
+    }
 
     // NOTE: we need to load autoproperties after decor have been updated
     // as otherwise old theme data pointer will be used and sig 11 pekwm.
-    list<Frame*>::iterator f_it(Frame::frame_begin());
-    for (; f_it != Frame::frame_end(); ++f_it) {
-        (*f_it)->readAutoprops();
+    list<Frame*>::iterator it(Frame::frame_begin());
+    for (; it != Frame::frame_end(); ++it) {
+        (*it)->readAutoprops();
     }
+}
 
 #ifdef HARBOUR
+/**
+ * Reload harbour configuration.
+ */
+void
+WindowManager::doReloadHarbour(void)
+{
     _harbour->loadTheme();
-    if (old_harbour_placement != _config->getHarbourPlacement()) {
-        _harbour->rearrange();
-    }
-    if (old_harbour_stacking != _config->isHarbourOntop()) {
-        _harbour->restack();
-    }
-    if (old_harbour_head_nr != _config->getHarbourHead()) {
-        _harbour->rearrange();
-    }
+    _harbour->rearrange();
+    _harbour->restack();
     _harbour->updateHarbourSize();
-#endif // HARBOUR
-
-#ifdef MENUS
-    updateMenus();
-#endif // MENUS
-
-    _screen->updateStrut();
-
-    _reload = false;
 }
+#endif // HARBOUR
 
 //! @brief Exit pekwm and restart with the command command
 void
@@ -1079,7 +1126,7 @@ WindowManager::doEventLoop(void)
 void
 WindowManager::handleKeyEvent(XKeyEvent *ev)
 {
-    ActionEvent *ae =	NULL;
+    ActionEvent *ae = 0;
     PWinObj *wo, *wo_orig;
     wo = wo_orig = PWinObj::getFocusedPWinObj();
     PWinObj::Type type = (wo) ? wo->getType() : PWinObj::WO_SCREEN_ROOT;
@@ -1133,7 +1180,11 @@ WindowManager::handleKeyEvent(XKeyEvent *ev)
 
     // flush Enter events caused by keygrabbing
     XEvent e;
-    while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, &e));
+    while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, &e)) {
+        if (! e.xcrossing.send_event) {
+            _screen->setLastEventTime(e.xcrossing.time);
+        }
+    }
 }
 
 //! @brief
@@ -1141,10 +1192,14 @@ void
 WindowManager::handleButtonPressEvent(XButtonEvent *ev)
 {
     // Clear event queue
-    while (XCheckTypedEvent(_screen->getDpy(), ButtonPress, (XEvent *) ev));
+    while (XCheckTypedEvent(_screen->getDpy(), ButtonPress, (XEvent *) ev)) {
+        if (! ev->send_event) {
+            _screen->setLastEventTime(ev->time);
+        }
+    }
 
-    ActionEvent *ae = NULL;
-    PWinObj *wo = NULL;
+    ActionEvent *ae = 0;
+    PWinObj *wo = 0;
 
     wo = PWinObj::findPWinObj(ev->window);
     if (wo) {
@@ -1185,9 +1240,13 @@ void
 WindowManager::handleButtonReleaseEvent(XButtonEvent *ev)
 {
     // Flush ButtonReleases
-    while (XCheckTypedEvent(_screen->getDpy(), ButtonRelease, (XEvent *) ev));
+    while (XCheckTypedEvent(_screen->getDpy(), ButtonRelease, (XEvent *) ev)) {
+        if (! ev->send_event) {
+            _screen->setLastEventTime(ev->time);
+        }
+    }
 
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
     PWinObj *wo = PWinObj::findPWinObj(ev->window);
 
     if (wo) {
@@ -1261,7 +1320,7 @@ WindowManager::handleConfigureRequestEvent(XConfigureRequestEvent *ev)
 void
 WindowManager::handleMotionEvent(XMotionEvent *ev)
 {
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
     PWinObj *wo = PWinObj::findPWinObj(ev->window);
 
     if (wo) {
@@ -1345,7 +1404,7 @@ void
 WindowManager::handleUnmapEvent(XUnmapEvent *ev)
 {
     PWinObj *wo = PWinObj::findPWinObj(ev->window);
-    PWinObj *wo_search = NULL;
+    PWinObj *wo_search = 0;
 
     PWinObj::Type wo_type = PWinObj::WO_NO_TYPE;
 
@@ -1358,7 +1417,7 @@ WindowManager::handleUnmapEvent(XUnmapEvent *ev)
         wo->handleUnmapEvent(ev);
 
         if (wo == PWinObj::getFocusedPWinObj()) {
-            PWinObj::setFocusedPWinObj(NULL);
+            PWinObj::setFocusedPWinObj(0);
         }
     }
 
@@ -1407,7 +1466,11 @@ void
 WindowManager::handleEnterNotify(XCrossingEvent *ev)
 {
     // Clear event queue
-    while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, (XEvent *) ev));
+    while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, (XEvent *) ev)) {
+        if (! ev->send_event) {
+            _screen->setLastEventTime(ev->time);
+        }
+    }
 
     if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab)) {
         return;
@@ -1437,7 +1500,11 @@ void
 WindowManager::handleLeaveNotify(XCrossingEvent *ev)
 {
     // Clear event queue
-    while (XCheckTypedEvent(_screen->getDpy(), LeaveNotify, (XEvent *) ev));
+    while (XCheckTypedEvent(_screen->getDpy(), LeaveNotify, (XEvent *) ev)) {
+        if (! ev->send_event) {
+            _screen->setLastEventTime(ev->time);
+        }
+    }
 
     if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab)) {
         return;
@@ -1466,9 +1533,6 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
         return;
     }
 
-    // Get the last focus in event, no need to go through them all.
-    while (XCheckTypedEvent(_screen->getDpy(), FocusIn, (XEvent *) ev));
-
     PWinObj *wo = PWinObj::findPWinObj(ev->window);
     if (wo) {
         // To simplify logic, changing client in frame wouldn't update the
@@ -1478,15 +1542,23 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
         }
 
         if (! wo->isFocusable() || ! wo->isMapped()) {
-            findWOAndFocus(NULL);
+            findWOAndFocus(0);
 
         } else if (wo != PWinObj::getFocusedPWinObj()) {
             // If it's a valid FocusIn event with accepatable target lets flush
             // all EnterNotify and LeaveNotify as they can interfere with
             // focusing if Sloppy or Follow like focus model is used.
             XEvent e_flush;
-            while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, &e_flush));
-            while (XCheckTypedEvent(_screen->getDpy(), LeaveNotify, &e_flush));
+            while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, &e_flush)) {
+                if (! e_flush.xcrossing.send_event) {
+                    _screen->setLastEventTime(e_flush.xcrossing.time);
+                }
+            }
+            while (XCheckTypedEvent(_screen->getDpy(), LeaveNotify, &e_flush)) {
+                if (! e_flush.xcrossing.send_event) {
+                    _screen->setLastEventTime(e_flush.xcrossing.time);
+                }
+            }
 
             PWinObj *focused_wo = PWinObj::getFocusedPWinObj(); // convenience
 
@@ -1506,16 +1578,13 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
                 wo->getParent()->setFocused(true);
                 setEwmhActiveWindow(wo->getWindow());
 
+                // update the MRU list
+                _mru_list.remove(wo->getParent());
+                _mru_list.push_back(wo->getParent());
             } else {
                 wo->setFocused(true);
                 setEwmhActiveWindow(None);
             }
-
-            // update the MRU list
-            if (wo->getType() == PWinObj::WO_CLIENT) {
-                _mru_list.remove(wo->getParent());
-                _mru_list.push_back(wo->getParent());
-            }
         }
     }
 }
@@ -1524,27 +1593,9 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
 void
 WindowManager::handleFocusOutEvent(XFocusChangeEvent *ev)
 {
-    if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab)) {
-        return;
-    }
-
     // Get the last focus in event, no need to go through them all.
     while (XCheckTypedEvent(_screen->getDpy(), FocusOut, (XEvent *) ev))
         ;
-
-    // Match against focusd PWinObj if any, if matches we see if
-    // there are any FocusIn events in the queue, if not we give focus to
-    // the root window as we don't want to get left without focus.
-    if (PWinObj::getFocusedPWinObj()
-            && (*PWinObj::getFocusedPWinObj() == ev->window)) {
-        if (XCheckTypedEvent(_screen->getDpy(), FocusIn, (XEvent *) ev)) {
-            // We found a Event, put it back and handle it later on.
-            XPutBackEvent(_screen->getDpy(), (XEvent *) ev);
-        } else {
-            // No event found, give the root PWinObj focus.
-            PWinObj::getRootPWinObj()->giveInputFocus();
-        }
-    }
 }
 
 void
@@ -1592,6 +1643,10 @@ void
 WindowManager::handlePropertyEvent(XPropertyEvent *ev)
 {
     if (ev->window == _screen->getRoot()) {
+        if (ev->atom == _ewmh_atoms->getAtom(NET_DESKTOP_NAMES)) {
+            readDesktopNamesHint();
+        }
+
         return;
     }
 
@@ -1615,7 +1670,7 @@ WindowManager::handleMappingEvent(XMappingEvent *ev)
 void
 WindowManager::handleExposeEvent(XExposeEvent *ev)
 {
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
 
     PWinObj *wo = PWinObj::findPWinObj(ev->window);
     if (wo) {
@@ -1733,8 +1788,9 @@ WindowManager::createMenus(void)
 
     // Load configuration, pass specific section to loading
     CfgParser menu_cfg;
-    if (menu_cfg.parse(_config->getMenuFile())
-            || menu_cfg.parse (string(SYSCONFDIR "/menu"))) {
+    if (menu_cfg.parse(_config->getMenuFile()) || menu_cfg.parse (string(SYSCONFDIR "/menu"))) {
+        // Make sure mtime gets updated
+        Util::requireReload(_menu_path, _config->getMenuFile(), _menu_mtime);
 
         // Load standard menus
         map<string, PMenu*>::iterator it = _menu_map.begin();
@@ -1743,14 +1799,21 @@ WindowManager::createMenus(void)
         }
 
         // Load standalone menus
-        updateMenusStandalone(menu_cfg.get_entry_root()->get_entry_next());
+        updateMenusStandalone(menu_cfg.get_entry_root());
     }
 }
 
-//! @brief (re)loads the menus in the menu configuration
+/**
+ * (re)loads the menus in the menu configuration if the file has been
+ * updated since last load.
+ */
 void
-WindowManager::updateMenus(void)
+WindowManager::doReloadMenus(void)
 {
+    if (! Util::requireReload(_menu_path, _config->getMenuFile(), _menu_mtime)) {
+        return;
+    }    
+
     // Load configuration, pass specific section to loading
     bool cfg_ok = true;
     CfgParser menu_cfg;
@@ -1767,7 +1830,6 @@ WindowManager::updateMenus(void)
             delete it->second;
             _menu_map.erase(it);
         } else {
-            it->second->loadDecor();
             // Only reload the menu if we got a ok configuration
             if (cfg_ok) {
                 it->second->reload(menu_cfg.get_entry_root()->find_section(it->second->getName()));
@@ -1776,26 +1838,26 @@ WindowManager::updateMenus(void)
     }
 
     // Update standalone root menus (name != ROOTMENU)
-    updateMenusStandalone(menu_cfg.get_entry_root()->get_entry_next());
+    updateMenusStandalone(menu_cfg.get_entry_root());
 
     // Special case for HARBOUR menu which is not included in the menu map
 #ifdef HARBOUR
-    _harbour->getHarbourMenu()->loadDecor();
     _harbour->getHarbourMenu()->reload(static_cast<CfgParser::Entry*>(0));
 #endif // HARBOUR
 }
 
 //! @brief Updates standalone root menus
 void
-WindowManager::updateMenusStandalone(CfgParser::Entry *cfg_root)
+WindowManager::updateMenusStandalone(CfgParser::Entry *section)
 {
     // Temporary name, as names are stored uppercase
     string menu_name;
 
     // Go through all but reserved section names and create menus
-    for (CfgParser::Entry *it = cfg_root; it; it = it->get_section_next ()) {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
         // Uppercase name
-        menu_name = it->get_name();
+        menu_name = (*it)->get_name();
         Util::to_upper(menu_name);
 
         // Create new menus, if the name is not reserved and not used
@@ -1804,9 +1866,8 @@ WindowManager::updateMenusStandalone(CfgParser::Entry *cfg_root)
                             menu_name)
                 && ! getMenu(menu_name)) {
             // Create, parse and add to map
-            PMenu *menu = new ActionMenu(ROOTMENU_STANDALONE_TYPE,
-                                         L"", menu_name);
-            menu->reload(it);
+            PMenu *menu = new ActionMenu(ROOTMENU_STANDALONE_TYPE, L"", menu_name);
+            menu->reload((*it)->get_section());
             _menu_map[menu->getName()] = menu;
         }
     }
@@ -1828,7 +1889,7 @@ WindowManager::deleteMenus(void)
 void
 WindowManager::findWOAndFocus(PWinObj *search)
 {
-    PWinObj *focus = NULL;
+    PWinObj *focus = 0;
 
     if (PWinObj::windowObjectExists(search) &&
             (search->isMapped()) && (search->isFocusable()))  {
@@ -1917,7 +1978,7 @@ WindowManager::familyRaiseLower(Client *client, bool raise)
     list<Client*>::iterator it(client_list.begin());
     for (; it != client_list.end(); ++it) {
         frame = dynamic_cast<Frame*>((*it)->getParent());
-        if (frame && frame->getActiveChild() == *it) {
+        if (frame) {
             if (raise) {
                 frame->raise();
             } else {
@@ -1939,10 +2000,10 @@ Frame*
 WindowManager::findGroup(AutoProperty *property)
 {
     if (! _allow_grouping) {
-        return NULL;
+        return 0;
     }
 
-    Frame *frame = NULL;
+    Frame *frame = 0;
 
 #define MATCH_GROUP(F,P) \
 ((P->group_global || ((F)->isMapped())) && \
@@ -2036,12 +2097,11 @@ Frame*
 WindowManager::getNextFrame(Frame* frame, bool mapped, uint mask)
 {
     if (! frame || (Frame::frame_size() < 2)) {
-        return NULL;
+        return 0;
     }
 
-    Frame *next_frame = NULL;
-    list<Frame*>::iterator f_it(find(Frame::frame_begin(), Frame::frame_end(),
-                                     frame));
+    Frame *next_frame = 0;
+    list<Frame*>::iterator f_it(find(Frame::frame_begin(), Frame::frame_end(), frame));
 
     if (f_it != Frame::frame_end()) {
         list<Frame*>::iterator n_it(f_it);
@@ -2072,12 +2132,11 @@ Frame*
 WindowManager::getPrevFrame(Frame* frame, bool mapped, uint mask)
 {
     if (! frame || (Frame::frame_size() < 2)) {
-        return NULL;
+        return 0;
     }
 
-    Frame *next_frame = NULL;
-    list<Frame*>::iterator f_it(find(Frame::frame_begin(), Frame::frame_end(),
-                                     frame));
+    Frame *next_frame = 0;
+    list<Frame*>::iterator f_it(find(Frame::frame_begin(), Frame::frame_end(), frame));
 
     if (f_it != Frame::frame_end()) {
         list<Frame*>::iterator n_it(f_it);
@@ -2109,14 +2168,42 @@ WindowManager::getPrevFrame(Frame* frame, bool mapped, uint mask)
 void
 WindowManager::hideAllMenus(void)
 {
+    bool do_focus = false;
+    PWinObj *wo = PWinObj::getFocusedPWinObj();
+    if (wo && wo->getType() == PWinObj::WO_MENU) {
+        do_focus = true;
+    }
+
     map<std::string, PMenu*>::iterator it(_menu_map.begin());
     for (; it != _menu_map.end(); ++it) {
         it->second->unmapAll();
     }
+
+    if (do_focus) {
+        findWOAndFocus(0);
+    }
 }
 #endif // MENUS
 // here follows methods for hints and atoms
 
+/**
+ * Reads the _NET_DESKTOP_NAMES hint and sets the workspaces names accordingly.
+ */
+void
+WindowManager::readDesktopNamesHint(void)
+{
+    uchar *data;
+    ulong data_length;
+    if (AtomUtil::getProperty(_screen->getRoot(), _ewmh_atoms->getAtom(NET_DESKTOP_NAMES),
+                              _ewmh_atoms->getAtom(UTF8_STRING),
+                              EXPECTED_DESKTOP_NAMES_LENGTH, &data, &data_length)) {
+        Config::instance()->setDesktopNamesUTF8(reinterpret_cast<char *>(data), data_length);
+        _workspaces->setNames();
+
+        XFree(data);
+    }
+}
+
 void
 WindowManager::initHints(void)
 {
@@ -2169,3 +2256,20 @@ WindowManager::setEwmhActiveWindow(Window w)
 {
     AtomUtil::setWindow(_screen->getRoot(), _ewmh_atoms->getAtom(NET_ACTIVE_WINDOW), w);
 }
+
+//! @brief Sets the _NET_DESKTOP_NAMES extended window manager hint.
+void
+WindowManager::setDesktopNames(void)
+{
+    unsigned char *desktopnames = 0;
+    unsigned int length = 0;
+    Config::instance()->getDesktopNamesUTF8(&desktopnames, &length);
+
+    if (desktopnames) {
+        XChangeProperty(PScreen::instance()->getDpy(), _screen->getRoot(),
+                        _ewmh_atoms->getAtom(NET_DESKTOP_NAMES),
+                        _ewmh_atoms->getAtom(UTF8_STRING), 8, PropModeReplace,
+                        desktopnames, length);
+        delete [] desktopnames;
+    }
+}
