@@ -1,6 +1,6 @@
 //
 // main.cc for pekwm
-// Copyright (C) 2003-2004 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright © 2003-2008 Claes Nästén <me@pekdon.net>
 //
 // main.cc for aewm++
 // Copyright (C) 2000 Frank Hale <frankhale@yahoo.com>
@@ -10,10 +10,14 @@
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "Client.hh"
+#include "Compat.hh"
 #include "Frame.hh"
 #include "WindowManager.hh"
 #include "Util.hh"
@@ -81,7 +85,7 @@ main(int argc, char **argv)
     for (int i = 1; i < argc; ++i)	{
         if ((strcmp("--display", argv[i]) == 0) && ((i + 1) < argc)) {
             setenv("DISPLAY", argv[++i], 1);
-        }	else if ((strcmp("--config", argv[i]) == 0) && ((i + 1) < argc)) {
+        } else if ((strcmp("--config", argv[i]) == 0) && ((i + 1) < argc)) {
             config_file = argv[++i];
         } else if (strcmp("--version", argv[i]) == 0) {
             Info::printVersion();
@@ -95,9 +99,14 @@ main(int argc, char **argv)
         }
     }
 
-    // read in the environment
-    if ((config_file.size() == 0) && getenv("PEKWM_CONFIG_FILE")) {
-        config_file = getenv("PEKWM_CONFIG_FILE");
+    // Get configuration file if none was specified as a parameter,
+    // default to reading environment, if not set get ~/.pekwm/config
+    if (config_file.size() == 0) {
+        if (getenv("PEKWM_CONFIG_FILE") && strlen(getenv("PEKWM_CONFIG_FILE"))) {
+            config_file = getenv("PEKWM_CONFIG_FILE");
+        } else {
+            config_file = string(getenv("HOME")) + string("/.pekwm/config");
+        }
     }
 
 #ifdef DEBUG
@@ -113,10 +122,9 @@ main(int argc, char **argv)
 
         // cleanup before restarting
         WindowManager::destroy();
-
         Util::iconv_deinit();
 
-        execlp("/bin/sh", "sh" , "-c", restart_command.c_str(), (char*) NULL);
+        execlp("/bin/sh", "sh" , "-c", restart_command.c_str(), (char*) 0);
     }
 
     WindowManager::destroy();
